#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: ec2_vpc_nacl
short_description: create and delete Network ACLs
version_added: 1.0.0
version_added_collection: community.aws
description:
  - Read the AWS documentation for Network ACLS
    U(https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_ACLs.html).
options:
  name:
    description:
      - Tagged name identifying a network ACL.
      - One and only one of the O(name) or O(nacl_id) is required.
      - Mutually exclusive with O(nacl_id).
    required: false
    type: str
  nacl_id:
    description:
      - NACL id identifying a network ACL.
      - One and only one of the O(name) or O(nacl_id) is required.
      - Mutually exclusive with O(name).
    required: false
    type: str
  vpc_id:
    description:
      - VPC id of the requesting VPC.
      - Required when state present.
    required: false
    type: str
  subnets:
    description:
      - The list of subnets that should be associated with the network ACL.
      - Must be specified as a list
      - Each subnet can be specified as subnet ID, or its tagged name.
    required: false
    type: list
    elements: str
    default: []
  egress:
    description:
      - A list of rules for outgoing traffic. Each rule must be specified as a list.
        Each rule may contain the rule number (integer 1-32766), protocol (one of ['tcp', 'udp', 'icmp', 'ipv6-icmp', '-1', 'all']),
        the rule action ('allow' or 'deny') the CIDR of the IPv4 or IPv6 network range to allow or deny,
        the ICMP type (-1 means all types), the ICMP code (-1 means all codes), the last port in the range for
        TCP or UDP protocols, and the first port in the range for TCP or UDP protocols.
        See examples.
    default: []
    required: false
    type: list
    elements: list
  ingress:
    description:
      - List of rules for incoming traffic. Each rule must be specified as a list.
        Each rule may contain the rule number (integer 1-32766), protocol (one of ['tcp', 'udp', 'icmp', 'ipv6-icmp', '-1', 'all']),
        the rule action ('allow' or 'deny') the CIDR of the IPv4 or IPv6 network range to allow or deny,
        the ICMP type (-1 means all types), the ICMP code (-1 means all codes), the last port in the range for
        TCP or UDP protocols, and the first port in the range for TCP or UDP protocols.
        See examples.
    default: []
    required: false
    type: list
    elements: list
  state:
    description:
      - Creates or modifies an existing NACL.
      - Deletes a NACL and reassociates subnets to the default NACL.
    required: false
    type: str
    choices: ['present', 'absent']
    default: present
author:
  - Mike Mochan (@mmochan)
notes:
  - Support for I(purge_tags) was added in release 4.0.0.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
  - amazon.aws.tags
"""

EXAMPLES = r"""
# Complete example to create and delete a network ACL
# that allows SSH, HTTP and ICMP in, and all traffic out.
- name: "Create and associate production DMZ network ACL with DMZ subnets"
  amazon.aws.ec2_vpc_nacl:
    vpc_id: vpc-12345678
    name: prod-dmz-nacl
    region: ap-southeast-2
    subnets: ['prod-dmz-1', 'prod-dmz-2']
    tags:
      CostCode: CC1234
      Project: phoenix
      Description: production DMZ
    ingress:
      # rule no, protocol, allow/deny, cidr, icmp_type, icmp_code,
      #                                             port from, port to
      - [100, 'tcp', 'allow', '0.0.0.0/0', null, null, 22, 22]
      - [200, 'tcp', 'allow', '0.0.0.0/0', null, null, 80, 80]
      - [205, 'tcp', 'allow', '::/0', null, null, 80, 80]
      - [300, 'icmp', 'allow', '0.0.0.0/0', 0, 8]
      - [305, 'ipv6-icmp', 'allow', '::/0', 0, 8]
    egress:
      - [100, 'all', 'allow', '0.0.0.0/0', null, null, null, null]
      - [105, 'all', 'allow', '::/0', null, null, null, null]
    state: 'present'

- name: "Remove the ingress and egress rules - defaults to deny all"
  amazon.aws.ec2_vpc_nacl:
    vpc_id: vpc-12345678
    name: prod-dmz-nacl
    region: ap-southeast-2
    subnets:
      - prod-dmz-1
      - prod-dmz-2
    tags:
      CostCode: CC1234
      Project: phoenix
      Description: production DMZ
    state: present

- name: "Remove the NACL subnet associations and tags"
  amazon.aws.ec2_vpc_nacl:
    vpc_id: 'vpc-12345678'
    name: prod-dmz-nacl
    region: ap-southeast-2
    state: present

- name: "Delete nacl and subnet associations"
  amazon.aws.ec2_vpc_nacl:
    vpc_id: vpc-12345678
    name: prod-dmz-nacl
    state: absent

- name: "Delete nacl by its id"
  amazon.aws.ec2_vpc_nacl:
    nacl_id: acl-33b4ee5b
    state: absent
"""

RETURN = r"""
nacl_id:
  description: The id of the NACL (when creating or updating an ACL).
  returned: success
  type: str
  sample: "acl-123456789abcdef01"
"""

from typing import Any
from typing import Dict
from typing import List
from typing import Optional

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_network_acl
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_network_acl_entry
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_network_acl
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_network_acl_entry
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_network_acls
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_subnets
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import replace_network_acl_association
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule

# VPC-supported IANA protocol numbers
# http://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml
PROTOCOL_NUMBERS = {"all": -1, "icmp": 1, "tcp": 6, "udp": 17, "ipv6-icmp": 58}


# Utility methods
def icmp_present(entry: List[str]) -> bool:
    return len(entry) == 6 and entry[1] in ["icmp", "ipv6-icmp"] or entry[1] in [1, 58]


def subnets_changed(client, module: AnsibleAWSModule, nacl_id: str, subnets_ids: List[str]) -> bool:
    changed = False
    vpc_id = module.params.get("vpc_id")

    if not subnets_ids:
        default_nacl_id = find_default_vpc_nacl(client, vpc_id)
        # Find subnets by Network ACL ids
        network_acls = describe_network_acls(
            client, Filters=[{"Name": "association.network-acl-id", "Values": [nacl_id]}]
        )
        subnets = [
            association["SubnetId"]
            for nacl in network_acls
            for association in nacl["Associations"]
            if association["SubnetId"]
        ]
        changed = associate_nacl_to_subnets(client, module, default_nacl_id, subnets)
        return changed

    network_acls = describe_network_acls(client, NetworkAclIds=[nacl_id])
    current_subnets = [
        association["SubnetId"]
        for nacl in network_acls
        for association in nacl["Associations"]
        if association["SubnetId"]
    ]
    subnets_added = [subnet for subnet in subnets_ids if subnet not in current_subnets]
    subnets_removed = [subnet for subnet in current_subnets if subnet not in subnets_ids]

    if subnets_added:
        changed |= associate_nacl_to_subnets(client, module, nacl_id, subnets_added)
    if subnets_removed:
        default_nacl_id = find_default_vpc_nacl(client, vpc_id)
        changed |= associate_nacl_to_subnets(client, module, default_nacl_id, subnets_removed)

    return changed


def nacls_changed(client, module: AnsibleAWSModule, nacl_info: Dict[str, Any]) -> bool:
    changed = False
    entries = nacl_info["Entries"]
    nacl_id = nacl_info["NetworkAclId"]
    aws_egress_rules = [rule for rule in entries if rule["Egress"] is True and rule["RuleNumber"] < 32767]
    aws_ingress_rules = [rule for rule in entries if rule["Egress"] is False and rule["RuleNumber"] < 32767]

    # Egress Rules
    changed |= rules_changed(client, nacl_id, module.params.get("egress"), aws_egress_rules, True, module.check_mode)
    # Ingress Rules
    changed |= rules_changed(client, nacl_id, module.params.get("ingress"), aws_ingress_rules, False, module.check_mode)
    return changed


def tags_changed(client, module: AnsibleAWSModule, nacl_id: str) -> bool:
    tags = module.params.get("tags")
    name = module.params.get("name")
    purge_tags = module.params.get("purge_tags")

    if name is None and tags is None:
        return False

    if module.params.get("tags") is None:
        # Only purge tags if tags is explicitly set to {} and purge_tags is True
        purge_tags = False

    new_tags = dict()
    if module.params.get("name") is not None:
        new_tags["Name"] = module.params.get("name")
    new_tags.update(module.params.get("tags") or {})

    return ensure_ec2_tags(
        client, module, nacl_id, tags=new_tags, purge_tags=purge_tags, retry_codes=["InvalidNetworkAclID.NotFound"]
    )


def ansible_to_boto3_dict_rule(ansible_rule: List[Any], egress: bool) -> Dict[str, Any]:
    boto3_rule = {}
    if isinstance(ansible_rule, list):
        boto3_rule["RuleNumber"] = ansible_rule[0]
        boto3_rule["Protocol"] = str(PROTOCOL_NUMBERS[ansible_rule[1]])
        boto3_rule["RuleAction"] = ansible_rule[2]
        boto3_rule["Egress"] = egress
        if is_ipv6(ansible_rule[3]):
            boto3_rule["Ipv6CidrBlock"] = ansible_rule[3]
        else:
            boto3_rule["CidrBlock"] = ansible_rule[3]
        if icmp_present(ansible_rule):
            boto3_rule["IcmpTypeCode"] = {"Type": int(ansible_rule[4]), "Code": int(ansible_rule[5])}
        else:
            if ansible_rule[6] or ansible_rule[7]:
                boto3_rule["PortRange"] = {"From": ansible_rule[6], "To": ansible_rule[7]}
    return boto3_rule


def find_added_rules(rules_a: List[Dict[str, Any]], rules_b: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
    results = []
    # A rule is considered as a new rule if either the RuleNumber does exist in the list of
    # current Rules stored in AWS or if the Rule differs with the Rule stored in AWS with the same RuleNumber
    for a in rules_a:
        if not any(a["RuleNumber"] == b["RuleNumber"] and a == b for b in rules_b):
            results.append(a)
    return results


def rules_changed(
    client,
    nacl_id: str,
    ansible_rules: List[List[str]],
    aws_rules: List[Dict[str, Any]],
    egress: bool,
    check_mode: bool,
) -> bool:
    # transform rules: from ansible list to boto3 dict
    ansible_rules = [ansible_to_boto3_dict_rule(r, egress) for r in ansible_rules]

    # find added rules
    added_rules = find_added_rules(ansible_rules, aws_rules)
    # find removed rules
    removed_rules = find_added_rules(aws_rules, ansible_rules)

    changed = False
    for rule in added_rules:
        changed = True
        if not check_mode:
            rule_number = rule.pop("RuleNumber")
            protocol = rule.pop("Protocol")
            rule_action = rule.pop("RuleAction")
            egress = rule.pop("Egress")
            create_network_acl_entry(
                client,
                network_acl_id=nacl_id,
                protocol=protocol,
                egress=egress,
                rule_action=rule_action,
                rule_number=rule_number,
                **rule,
            )

    # Removed Rules
    for rule in removed_rules:
        changed = True
        if not check_mode:
            delete_network_acl_entry(client, network_acl_id=nacl_id, rule_number=rule["RuleNumber"], egress=egress)

    return changed


def is_ipv6(cidr: str) -> bool:
    return ":" in cidr


def process_rule_entry(entry: List[Any]) -> Dict[str, Any]:
    params = {}
    if is_ipv6(entry[3]):
        params["Ipv6CidrBlock"] = entry[3]
    else:
        params["CidrBlock"] = entry[3]
    if icmp_present(entry):
        params["IcmpTypeCode"] = {"Type": int(entry[4]), "Code": int(entry[5])}
    else:
        if entry[6] or entry[7]:
            params["PortRange"] = {"From": entry[6], "To": entry[7]}

    return params


def add_network_acl_entries(
    client, nacl_id: str, ansible_entries: List[List[str]], egress: bool, check_mode: bool
) -> bool:
    changed = False
    for entry in ansible_entries:
        changed = True
        if not check_mode:
            create_network_acl_entry(
                client,
                network_acl_id=nacl_id,
                protocol=str(PROTOCOL_NUMBERS[entry[1]]),
                egress=egress,
                rule_action=entry[2],
                rule_number=entry[0],
                **process_rule_entry(entry),
            )
    return changed


def associate_nacl_to_subnets(client, module: AnsibleAWSModule, nacl_id: str, subnets_ids: List[str]) -> bool:
    changed = False
    if subnets_ids:
        network_acls = describe_network_acls(client, Filters=[{"Name": "association.subnet-id", "Values": subnets_ids}])
        associations = [
            association["NetworkAclAssociationId"]
            for nacl in network_acls
            for association in nacl["Associations"]
            if association["SubnetId"] in subnets_ids
        ]
        for association_id in associations:
            changed = True
            if not module.check_mode:
                replace_network_acl_association(client, network_acl_id=nacl_id, association_id=association_id)
    return changed


def ensure_present(client, module: AnsibleAWSModule) -> None:
    changed = False
    nacl = describe_network_acl(client, module)
    nacl_id = None
    subnets_ids = []
    subnets = module.params.get("subnets")
    if subnets:
        subnets_ids = find_subnets_ids(client, module, subnets)

    if not nacl:
        if module.check_mode:
            module.exit_json(changed=True, msg="Would have created Network ACL if not in check mode.")

        # Create Network ACL
        tags = {}
        name = module.params.get("name")
        vpc_id = module.params.get("vpc_id")
        if name:
            tags["Name"] = name
        if module.params.get("tags"):
            tags.update(module.params.get("tags"))
        nacl = create_network_acl(client, vpc_id, tags)
        changed = True

        # Associate Subnets to Network ACL
        nacl_id = nacl["NetworkAclId"]
        changed |= associate_nacl_to_subnets(client, module, nacl_id, subnets_ids)

        # Create Network ACL entries (ingress and egress)
        changed |= add_network_acl_entries(
            client, nacl_id, module.params.get("ingress"), egress=False, check_mode=module.check_mode
        )
        changed |= add_network_acl_entries(
            client, nacl_id, module.params.get("egress"), egress=True, check_mode=module.check_mode
        )
    else:
        nacl_id = nacl["NetworkAclId"]
        changed |= subnets_changed(client, module, nacl_id, subnets_ids)
        changed |= nacls_changed(client, module, nacl)
        changed |= tags_changed(client, module, nacl_id)

    module.exit_json(changed=changed, nacl_id=nacl_id)


def ensure_absent(client, module: AnsibleAWSModule) -> None:
    changed = False
    result = {}
    nacl = describe_network_acl(client, module)
    if not nacl:
        module.exit_json(changed=changed)

    nacl_id = nacl["NetworkAclId"]
    vpc_id = nacl["VpcId"]
    associations = nacl["Associations"]
    assoc_ids = [a["NetworkAclAssociationId"] for a in associations]

    # Find default NACL associated to the VPC
    default_nacl_id = find_default_vpc_nacl(client, vpc_id)
    if not default_nacl_id:
        module.exit_json(changed=changed, msg="Default NACL ID not found - Check the VPC ID")

    # Replace Network ACL association
    for assoc_id in assoc_ids:
        changed = True
        if not module.check_mode:
            replace_network_acl_association(client, network_acl_id=default_nacl_id, association_id=assoc_id)

    # Delete Network ACL
    changed = True
    if module.check_mode:
        module.exit_json(changed=changed, msg=f"Would have deleted Network ACL id '{nacl_id}' if not in check mode.")

    changed = delete_network_acl(client, network_acl_id=nacl_id)
    module.exit_json(changed=changed, msg=f"Network ACL id '{nacl_id}' successfully deleted.")


def describe_network_acl(client, module: AnsibleAWSModule) -> Optional[Dict[str, Any]]:
    nacl_id = module.params.get("nacl_id")
    name = module.params.get("name")

    if nacl_id:
        filters = [{"Name": "network-acl-id", "Values": [nacl_id]}]
    else:
        filters = [{"Name": "tag:Name", "Values": [name]}]
    network_acls = describe_network_acls(client, Filters=filters)
    return None if not network_acls else network_acls[0]


def find_default_vpc_nacl(client, vpc_id: str) -> Optional[str]:
    default_nacl_id = None
    for nacl in describe_network_acls(client, Filters=[{"Name": "vpc-id", "Values": [vpc_id]}]):
        if nacl.get("IsDefault", False):
            default_nacl_id = nacl["NetworkAclId"]
            break
    return default_nacl_id


def find_subnets_ids(client, module: AnsibleAWSModule, subnets_ids_or_names: List[str]) -> List[str]:
    subnets_ids = []
    subnets_names = []

    # Find Subnets by ID
    subnets = describe_subnets(client, Filters=[{"Name": "subnet-id", "Values": subnets_ids_or_names}])
    subnets_ids += [subnet["SubnetId"] for subnet in subnets]
    subnets_names += [tag["Value"] for subnet in subnets for tag in subnet.get("Tags", []) if tag["Key"] == "Name"]

    # Find Subnets by Name
    subnets = describe_subnets(client, Filters=[{"Name": "tag:Name", "Values": subnets_ids_or_names}])
    subnets_ids += [subnet["SubnetId"] for subnet in subnets]
    subnets_names += [tag["Value"] for subnet in subnets for tag in subnet.get("Tags", []) if tag["Key"] == "Name"]

    unexisting_subnets = [s for s in subnets_ids_or_names if s not in subnets_names + subnets_ids]
    if unexisting_subnets:
        module.fail_json(msg=f"The following subnets do not exist: {unexisting_subnets}")
    return subnets_ids


def main():
    argument_spec = dict(
        vpc_id=dict(),
        name=dict(),
        nacl_id=dict(),
        subnets=dict(required=False, type="list", default=[], elements="str"),
        tags=dict(required=False, type="dict", aliases=["resource_tags"]),
        purge_tags=dict(required=False, type="bool", default=True),
        ingress=dict(required=False, type="list", default=list(), elements="list"),
        egress=dict(required=False, type="list", default=list(), elements="list"),
        state=dict(default="present", choices=["present", "absent"]),
    )

    mutually_exclusive = [
        ["name", "nacl_id"],
    ]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=[["name", "nacl_id"]],
        required_if=[["state", "present", ["vpc_id"]]],
        mutually_exclusive=mutually_exclusive,
    )

    client = module.client("ec2")

    try:
        if module.params.get("state") == "present":
            ensure_present(client, module)
        else:
            ensure_absent(client, module)
    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: users_external_servers_aaa_attribute
short_description: Resource module for Users External Servers Aaa Attribute
description:
- This module represents an alias of the module users_external_servers_aaa_attribute_v1
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  attributeName:
    description: Name of the custom AAA attribute.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for User and Roles AddAndUpdateAAAAttributeAPIV1
  description: Complete reference of the AddAndUpdateAAAAttributeAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-and-update-aaa-attribute-api
- name: Cisco DNA Center documentation for User and Roles DeleteAAAAttributeAPIV1
  description: Complete reference of the DeleteAAAAttributeAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-aaa-attribute-api
notes:
  - SDK Method used are
    user_and_roles.UserandRoles.add_and_update_aaa_attribute_api_v1,
    user_and_roles.UserandRoles.delete_aaa_attribute_api_v1,

  - Paths used are
    post /dna/system/api/v1/users/external-servers/aaa-attribute,
    delete /dna/system/api/v1/users/external-servers/aaa-attribute,
  - It should be noted that this module is an alias of users_external_servers_aaa_attribute_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.users_external_servers_aaa_attribute:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    attributeName: string

- name: Delete all
  cisco.dnac.users_external_servers_aaa_attribute:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "message": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Anvitha Jain (@anvitha-jain) <anvjain@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
---
module: aci_cloud_external_epg_selector
short_description: Manage Cloud Endpoint Selector for External EPGs (cloud:ExtEPSelector)
description:
- Decides which endpoints belong to the EPGs based on several parameters.
options:
  name:
    description:
    - The name of the Cloud Endpoint selector.
    aliases: [ selector, cloud_external_epg_selector, external_epg_selector, extepg_selector, selector_name ]
    type: str
  subnet:
    description:
    - IP address of the Cloud Subnet.
    aliases: [ ip ]
    type: str
  tenant:
    description:
    - The name of tenant.
    type: str
  ap:
    description:
    - The name of the cloud application profile.
    aliases: [ app_profile, app_profile_name ]
    type: str
  cloud_external_epg:
    description:
    - Name of Object cloud_external_epg.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    choices: [ absent, present, query ]
    default: present
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- More information about the internal APIC class B(cloud:ExtEPSelector) from
  L(the APIC Management Information Model reference,https://developer.cisco.com/docs/apic-mim-ref/).
author:
- Anvitha Jain (@anvitha-jain)
"""

EXAMPLES = r"""
- name: Add a new cloud external EPG selector
  cisco.aci.aci_cloud_external_epg_selector:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: tenant1
    ap: ap1
    cloud_external_epg: ext_epg
    name: subnet_name
    subnet: 10.0.0.0/16
    state: present
  delegate_to: localhost

- name: Remove a cloud external EPG selector
  cisco.aci.aci_cloud_external_epg_selector:
    host: apic
    username: admin
    password: SomeSecretPassword
    validate_certs: false
    tenant: tenant1
    ap: ap1
    cloud_external_epg: ext_epg
    name: subnet_name
    subnet: 10.0.0.0/16
    state: absent
  delegate_to: localhost

- name: Query all cloud external EPG selectors
  cisco.aci.aci_cloud_external_epg_selector:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: tenant1
    ap: ap1
    cloud_external_epg: ext_epg
    state: query
  delegate_to: localhost
"""

from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible.module_utils.basic import AnsibleModule


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        {
            "name": dict(type="str", aliases=["selector", "cloud_external_epg_selector", "external_epg_selector", "extepg_selector", "selector_name"]),
            "subnet": dict(type="str", aliases=["ip"]),
            "tenant": dict(type="str"),
            "cloud_external_epg": dict(type="str"),
            "ap": dict(type="str", aliases=["app_profile", "app_profile_name"]),
            "state": dict(type="str", default="present", choices=["absent", "present", "query"]),
        }
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["subnet", "tenant", "ap", "cloud_external_epg"]],
            ["state", "present", ["subnet", "tenant", "ap", "cloud_external_epg"]],
        ],
    )

    name = module.params.get("name")
    subnet = module.params.get("subnet")
    tenant = module.params.get("tenant")
    ap = module.params.get("ap")
    cloud_external_epg = module.params.get("cloud_external_epg")
    state = module.params.get("state")
    child_configs = []

    aci = ACIModule(module)
    aci.construct_url(
        root_class={
            "aci_class": "fvTenant",
            "aci_rn": "tn-{0}".format(tenant),
            "target_filter": 'eq(fvTenant.name, "{0}")'.format(tenant),
            "module_object": tenant,
        },
        subclass_1={"aci_class": "cloudApp", "aci_rn": "cloudapp-{0}".format(ap), "target_filter": 'eq(cloudApp.name, "{0}")'.format(ap), "module_object": ap},
        subclass_2={
            "aci_class": "cloudExtEPg",
            "aci_rn": "cloudextepg-{0}".format(cloud_external_epg),
            "target_filter": 'eq(cloudExtEPg.name, "{0}")'.format(cloud_external_epg),
            "module_object": cloud_external_epg,
        },
        subclass_3={
            "aci_class": "cloudExtEPSelector",
            "aci_rn": "extepselector-[{0}]".format(subnet),
            "target_filter": 'eq(cloudExtEPSelector.name, "{0}")'.format(subnet),
            "module_object": subnet,
        },
        child_classes=[],
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="cloudExtEPSelector",
            class_config={
                "name": name,
                "subnet": subnet,
            },
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="cloudExtEPSelector")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The parse_cli_textfsm plugin code
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import os

from ansible.errors import AnsibleFilterError
from ansible.module_utils._text import to_native
from ansible.module_utils.six import string_types


try:
    import textfsm

    HAS_TEXTFSM = True
except ImportError:
    HAS_TEXTFSM = False


def _raise_error(msg):
    raise AnsibleFilterError(msg)


def parse_cli_textfsm(value, template):
    if not HAS_TEXTFSM:
        _raise_error("parse_cli_textfsm filter requires TextFSM library to be installed")

    if not isinstance(value, string_types):
        _raise_error(
            "parse_cli_textfsm input should be a string, but was given a input of %s"
            % (type(value))
        )

    if not os.path.exists(template):
        _raise_error("unable to locate parse_cli_textfsm template: %s" % template)

    try:
        template = open(template)
    except IOError as exc:
        _raise_error(to_native(exc))

    re_table = textfsm.TextFSM(template)
    fsm_results = re_table.ParseText(value)

    results = list()
    for item in fsm_results:
        results.append(dict(zip(re_table.header, item)))

    return results



#
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type


from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import Facts
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.ospfv3 import (
    Ospfv3Template,
)


class Ospfv3(ResourceModule):
    """
    The ios_ospfv3 class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["ospfv3"]

    def __init__(self, module):
        super(Ospfv3, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="ospfv3",
            tmplt=Ospfv3Template(),
        )

    def execute_module(self):
        """Execute the module
        :rtype: A dictionary
        :returns: The result from module execution
        """
        self.gen_config()
        self.run_commands()

        return self.result

    def gen_config(self):
        """Select the appropriate function based on the state provided
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        if self.want:
            wantd = {
                (entry["process_id"], entry.get("vrf")): entry
                for entry in self.want.get("processes", [])
            }
        else:
            wantd = {}
        if self.have:
            haved = {
                (entry["process_id"], entry.get("vrf")): entry
                for entry in self.have.get("processes", [])
            }
        else:
            haved = {}

        # turn all lists of dicts into dicts prior to merge
        for thing in wantd, haved:
            for _pid, proc in iteritems(thing):
                for area in proc.get("areas", []):
                    virtual_link = {entry["id"]: entry for entry in area.get("virtual_link", [])}
                    if bool(virtual_link):
                        area["virtual_link"] = virtual_link
                    ranges = {entry["address"]: entry for entry in area.get("ranges", [])}
                    if bool(ranges):
                        area["ranges"] = ranges

                proc["areas"] = {entry["area_id"]: entry for entry in proc.get("areas", [])}
                if proc.get("distribute_list"):
                    if "acls" in proc.get("distribute_list"):
                        proc["distribute_list"]["acls"] = {
                            entry["name"]: entry
                            for entry in proc["distribute_list"].get(
                                "acls",
                                [],
                            )
                        }

        # if state is merged, merge want onto have
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, limit the have to anything in want
        # set want to nothing
        if self.state == "deleted":
            haved = {k: v for k, v in iteritems(haved) if k in wantd or not wantd}
            wantd = {}

        # if state is overridden, first remove processes that are in have but not in want
        if self.state in ["overridden", "deleted"]:
            for k, have in iteritems(haved):
                if k not in wantd:
                    self.addcmd(have, "pid", True)

        for k, want in iteritems(wantd):
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        parsers = [
            "bfd.minimum_interval",
            "bfd.multiplier",
            "cost",
            "weight",
            "passive",
            "priority",
            "protocol",
            "auto_cost",
            "bandwidth",
            "flood_reduction",
            "default_metric",
            "default_weight",
            "router_id",
            "demand_circuit",
            "packet_size",
            "transmit_delay",
            "dead_interval",
            "hello_interval",
            "authentication",
            "retransmit_interval",
            "mtu_ignore",
            "bfd.fast_detect",
            "capability",
            "capability.opaque",
            "admin_distance",
            "ospf_distance",
            "address_family_unicast",
            "loopback_stub_network",
            "authentication.ipsec",
            "default_information_originate",
            "link_down_fast_detect",
            "nsr",
            "database_filter",
            "log_adjacency",
            "distribute_bgp_ls",
            "distribute_link_state",
            "max_lsa",
            "max_metric",
            "mpls_ldp",
            "mpls_traffic_eng",
            "microloop_avoidance",
            "prefix_suppression",
            "protocol_shutdown",
            "timers.lsa",
            "timers.graceful_shutdown",
            "throttle.lsa_all",
            "throttle.spf",
            "throttle.fast_reroute",
            "timers.pacing_flood",
        ]

        if want != have:
            self.addcmd(want or have, "pid", False)
            self.compare(parsers, want, have)
            self._areas_compare(want, have)

    def _areas_compare(self, want, have):
        wareas = want.get("areas", {})
        hareas = have.get("areas", {})
        for name, entry in iteritems(wareas):
            self._area_compare(want=entry, have=hareas.pop(name, {}))
        for name, entry in iteritems(hareas):
            self._area_compare(want={}, have=entry)

    def _area_compare(self, want, have):
        parsers = [
            "area.authentication",
            "area.authentication.ipsec",
            "area.mpls_traffic_eng",
            "area.mpls_ldp",
            "area.bfd.minimum_interval",
            "area.bfd.multiplier",
            "area.bfd.fast_detect",
            "area.nssa",
            "area.nssa.default_information_originate",
            "area.nssa.translate",
            "area.default_cost",
            "area.stub",
            "area.ranges",
            "area.cost",
            "area.dead_interval",
            "area.hello_interval",
            "area.transmit_delay",
            "area.mtu_ignore",
            "area.packet_size",
            "area.priority",
            "area.weight",
            "area.external_out",
            "area.summary_in",
            "area.demand_circuit",
            "area.passive",
        ]
        self.compare(parsers=parsers, want=want, have=have)
        self._areas_compare_virtual_link(want, have)

    def _areas_compare_virtual_link(self, want, have):
        wvlinks = want.get("virtual_link", {})
        hvlinks = have.get("virtual_link", {})
        for name, entry in iteritems(wvlinks):
            self._area_compare_virtual_link(
                want=entry,
                have=hvlinks.pop(name, {}),
            )
        for name, entry in iteritems(hvlinks):
            self._area_compare_virtual_link(want={}, have=entry)

    def _area_compare_virtual_link(self, want, have):
        parsers = [
            "virtual_link.authentication",
            "virtual_link.authentication_key",
            "virtual_link.authentication.message_digest",
            "virtual_link.hello_interval",
            "virtual_link.dead_interval",
            "virtual_link.retransmit_interval",
        ]
        self.compare(parsers=parsers, want=want, have=have)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: ec2_transit_gateway
short_description: Create and delete AWS Transit Gateways
version_added: 1.0.0
version_added_collection: community.aws
description:
  - Creates AWS Transit Gateways.
  - Deletes AWS Transit Gateways.
  - Updates tags on existing transit gateways.
options:
  asn:
    description:
      - A private Autonomous System Number (ASN) for the Amazon side of a BGP session.
      - The range is 64512 to 65534 for 16-bit ASNs and 4200000000 to 4294967294 for 32-bit ASNs.
    type: int
  auto_associate:
    description:
      - Enable or disable automatic association with the default association route table.
    default: true
    type: bool
  auto_attach:
    description: Enable or disable automatic acceptance of attachment requests.
    default: false
    type: bool
  auto_propagate:
    description: Enable or disable automatic propagation of routes to the default propagation route table.
    default: true
    type: bool
  description:
     description: The description of the transit gateway.
     type: str
  dns_support:
    description: Whether to enable AWS DNS support.
    default: true
    type: bool
  multicast_support:
    description: Whether to enable AWS Multicast support. Valid only at the time of creation of the Transit Gateway.
    type: bool
    version_added: 8.1.0
  state:
    description:
      - V(present) to ensure resource is created.
      - V(absent) to remove resource.
    default: present
    choices: [ "present", "absent"]
    type: str
  transit_gateway_id:
    description: The ID of the transit gateway.
    type: str
  vpn_ecmp_support:
    description: Enable or disable Equal Cost Multipath Protocol support.
    default: true
    type: bool
  wait:
    description: Whether to wait for status.
    default: true
    type: bool
  wait_timeout:
    description: Number of seconds to wait for status.
    default: 300
    type: int

author:
  - "Bob Boldin (@BobBoldin)"
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
  - amazon.aws.tags
"""

EXAMPLES = r"""
- name: Create a new transit gateway using defaults
  amazon.aws.ec2_transit_gateway:
    state: present
    region: us-east-1
    description: personal-testing
  register: created_tgw

- name: Create a new transit gateway with options
  amazon.aws.ec2_transit_gateway:
    asn: 64514
    auto_associate: false
    auto_propagate: false
    multicast_support: true
    dns_support: true
    description: "nonprod transit gateway"
    purge_tags: false
    state: present
    region: us-east-1
    tags:
      Name: nonprod transit gateway
      status: testing

- name: Remove a transit gateway by description
  amazon.aws.ec2_transit_gateway:
    state: absent
    region: us-east-1
    description: personal-testing

- name: Remove a transit gateway by id
  amazon.aws.ec2_transit_gateway:
    state: absent
    region: ap-southeast-2
    transit_gateway_id: tgw-3a9aa123
  register: deleted_tgw
"""

RETURN = r"""
transit_gateway:
  description: The attributes of the transit gateway.
  type: dict
  returned: I(state=present)
  contains:
    creation_time:
      description: The creation time of the transit gateway.
      returned: always
      type: str
      sample: "2019-03-06T17:13:51+00:00"
    description:
      description: The description of the transit gateway.
      returned: always
      type: str
      sample: "my test tgw"
    options:
      description: The options attributes of the transit gateway.
      returned: always
      type: dict
      contains:
        amazon_side_asn:
          description:
            - A private Autonomous System Number (ASN) for the Amazon side of a BGP session.
              The range is 64512 to 65534 for 16-bit ASNs and 4200000000 to 4294967294 for 32-bit ASNs.
          returned: always
          type: str
          sample: "64512"
        auto_accept_shared_attachements:
          description: Indicates whether attachment requests are automatically accepted.
          returned: always
          type: str
          sample: "disable"
        default_route_table_association:
          description:
           - Indicates  whether resource attachments are automatically
              associated with the default association route table.
          returned: always
          type: str
          sample: "enable"
        association_default_route_table_id:
          description: The ID of the default association route table.
          returned: Iwhen exists
          type: str
          sample: "tgw-rtb-abc123444"
        default_route_table_propagation:
          description:
           - Indicates  whether  resource  attachments   automatically
             propagate routes to the default propagation route table.
          returned: always
          type: str
          sample: "disable"
        propagation_default_route_table_id:
          description: The ID of the default propagation route table.
          returned: when exists
          type: str
          sample: "tgw-rtb-def456777"
        vpn_ecmp_support:
          description: Indicates  whether  Equal Cost Multipath Protocol support is enabled.
          returned: always
          type: str
          sample: "enable"
        dns_support:
          description: Indicates whether DNS support is enabled.
          returned: always
          type: str
          sample: "enable"
        multicast_support:
          description: Indicates whether Multicast support is enabled.
          returned: always
          type: str
          sample: "enable"
          version_added: 7.3.0
    owner_id:
      description: The account that owns the transit gateway.
      returned: always
      type: str
      sample: "123456789012"
    state:
      description: The state of the transit gateway.
      returned: always
      type: str
      sample: "pending"
    tags:
      description: A dictionary of resource tags.
      returned: always
      type: dict
      sample:
        tags:
          Name: nonprod_tgw
    transit_gateway_arn:
      description: The ID of the transit_gateway.
      returned: always
      type: str
      sample: "tgw-3a9aa123"
    transit_gateway_id:
      description: The ID of the transit_gateway.
      returned: always
      type: str
      sample: "tgw-3a9aa123"
"""

from typing import Any
from typing import Dict
from typing import Optional

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_ec2_transit_gateway
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_ec2_transit_gateway
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_ec2_transit_gateways
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list
from ansible_collections.amazon.aws.plugins.module_utils.waiters import wait_for_resource_state


class AnsibleEc2Tgw:
    def __init__(self, module: AnsibleAWSModule, results: Dict[str, Any]) -> None:
        self._module = module
        self._results = results
        retry_decorator = AWSRetry.jittered_backoff(
            catch_extra_error_codes=["IncorrectState"],
        )
        self._connection = module.client("ec2")
        self._check_mode = self._module.check_mode

    def process(self) -> None:
        """Process the request based on state parameter."""
        description = self._module.params.get("description")
        state = self._module.params.get("state", "present")
        tgw_id = self._module.params.get("transit_gateway_id")

        if state == "present":
            self.ensure_tgw_present(tgw_id, description)
        elif state == "absent":
            self.ensure_tgw_absent(tgw_id, description)

    def wait_for_status(self, wait_timeout: int, tgw_id: str, status: str, skip_deleted: bool = True) -> Dict[str, Any]:
        """
        Wait for the Transit Gateway to reach the specified status.
        :param wait_timeout: Number of seconds to wait, until this timeout is reached.
        :param tgw_id: The Amazon NAT ID.
        :param status: The status to wait for.
        :param skip_deleted: Ignore deleted transit gateways.
        :return: Transit gateway object.
        """
        polling_increment_secs = 5
        max_attempts = wait_timeout // polling_increment_secs

        waiter_method = f"transit_gateway_{status}"

        wait_for_resource_state(
            self._connection,
            self._module,
            waiter_method,
            TransitGatewayIds=[tgw_id],
            delay=polling_increment_secs,
            max_attempts=max_attempts,
        )

        transit_gateway = self.get_matching_tgw(tgw_id=tgw_id, skip_deleted=skip_deleted)
        if transit_gateway is None:
            self._module.fail_json(msg="Transit Gateway not found after waiting.")

        return transit_gateway

    def get_matching_tgw(
        self, tgw_id: Optional[str], description: Optional[str] = None, skip_deleted: bool = True
    ) -> Optional[Dict[str, Any]]:
        """Search for an existing tgw by either tgw_id or description.
        :param tgw_id: The AWS id of the transit gateway.
        :param description: The description of the transit gateway.
        :param skip_deleted: Ignore deleted transit gateways.
        :return: Transit gateway object.
        """
        filters = []
        params = {}
        if tgw_id:
            filters = ansible_dict_to_boto3_filter_list({"transit-gateway-id": tgw_id})

        params["Filters"] = filters
        response = describe_ec2_transit_gateways(self._connection, **params)

        tgw = None
        tgws = []

        if len(response) == 1 and tgw_id:
            if (response[0]["State"] != "deleted") or not skip_deleted:
                tgws.extend(response)

        for gateway in response:
            if description == gateway["Description"] and gateway["State"] != "deleted":
                tgws.append(gateway)

        if len(tgws) > 1:
            self._module.fail_json(
                msg=f"EC2 returned more than one transit Gateway for description {description}, aborting"
            )
        elif tgws:
            tgw = camel_dict_to_snake_dict(tgws[0], ignore_list=["Tags"])
            tgw["tags"] = boto3_tag_list_to_ansible_dict(tgws[0]["Tags"])

        return tgw

    @staticmethod
    def enable_option_flag(flag: Optional[bool]) -> str:
        disabled = "disable"
        enabled = "enable"
        return enabled if flag else disabled

    def create_tgw(self, description: str) -> Dict[str, Any]:
        """
        Create a transit gateway and optionally wait for status to become available.
        :param description: The description of the transit gateway.
        :return: Transit gateway object.
        """
        options: Dict[str, Any] = {}
        wait = self._module.params.get("wait")
        wait_timeout = self._module.params.get("wait_timeout")

        if self._module.params.get("asn"):
            options["AmazonSideAsn"] = self._module.params.get("asn")

        options["AutoAcceptSharedAttachments"] = self.enable_option_flag(self._module.params.get("auto_attach"))
        options["DefaultRouteTableAssociation"] = self.enable_option_flag(self._module.params.get("auto_associate"))
        options["DefaultRouteTablePropagation"] = self.enable_option_flag(self._module.params.get("auto_propagate"))
        options["VpnEcmpSupport"] = self.enable_option_flag(self._module.params.get("vpn_ecmp_support"))
        options["DnsSupport"] = self.enable_option_flag(self._module.params.get("dns_support"))
        options["MulticastSupport"] = self.enable_option_flag(self._module.params.get("multicast_support"))

        params = {"Description": description, "Options": options}

        response = create_ec2_transit_gateway(self._connection, **params)

        tgw_id = response["TransitGatewayId"]

        if wait:
            result = self.wait_for_status(wait_timeout=wait_timeout, tgw_id=tgw_id, status="available")
        else:
            result = self.get_matching_tgw(tgw_id=tgw_id)

        self._results["msg"] = f"Transit gateway {result['transit_gateway_id']} created"

        return result

    def delete_tgw(self, tgw_id: str) -> Dict[str, Any]:
        """
        Delete the transit gateway and optionally wait for status to become deleted.
        :param tgw_id: The id of the transit gateway.
        :return: Transit gateway object.
        """
        wait = self._module.params.get("wait")
        wait_timeout = self._module.params.get("wait_timeout")

        delete_ec2_transit_gateway(self._connection, tgw_id)

        if wait:
            result = self.wait_for_status(
                wait_timeout=wait_timeout, tgw_id=tgw_id, status="deleted", skip_deleted=False
            )
        else:
            result = self.get_matching_tgw(tgw_id=tgw_id, skip_deleted=False)

        self._results["msg"] = f"Transit gateway {tgw_id} deleted"

        return result

    def ensure_tgw_present(self, tgw_id: Optional[str] = None, description: Optional[str] = None) -> Dict[str, Any]:
        """
        Will create a tgw if no match to the tgw_id or description are found.
        Will update the tgw tags if matching one found but tags are not synced.
        :param tgw_id: The AWS id of the transit gateway.
        :param description: The description of the transit gateway.
        :return: Transit gateway object.
        """
        tgw = self.get_matching_tgw(tgw_id, description)

        if tgw is None:
            if self._check_mode:
                self._results["changed"] = True
                self._results["transit_gateway_id"] = None
                return self._results

            if not description:
                self._module.fail_json(msg="Failed to create Transit Gateway: description argument required")
            tgw = self.create_tgw(description)
            self._results["changed"] = True

        self._results["changed"] |= ensure_ec2_tags(
            self._connection,
            self._module,
            tgw["transit_gateway_id"],
            tags=self._module.params.get("tags"),
            purge_tags=self._module.params.get("purge_tags"),
        )

        self._results["transit_gateway"] = self.get_matching_tgw(tgw_id=tgw["transit_gateway_id"])

        return self._results

    def ensure_tgw_absent(self, tgw_id: Optional[str] = None, description: Optional[str] = None) -> Dict[str, Any]:
        """
        Will delete the tgw if a single tgw is found not yet in deleted status.
        :param tgw_id: The AWS id of the transit gateway.
        :param description: The description of the transit gateway.
        :return: Transit gateway object.
        """
        self._results["transit_gateway_id"] = None
        tgw = self.get_matching_tgw(tgw_id, description)

        if tgw is not None:
            if self._check_mode:
                self._results["changed"] = True
                return self._results

            tgw = self.delete_tgw(tgw_id=tgw["transit_gateway_id"])
            self._results["changed"] = True
            self._results["transit_gateway"] = self.get_matching_tgw(
                tgw_id=tgw["transit_gateway_id"], skip_deleted=False
            )

        return self._results


def setup_module_object() -> AnsibleAWSModule:
    """
    merge argument spec and create Ansible module object
    :return: Ansible module object
    """

    argument_spec = dict(
        asn=dict(type="int"),
        auto_associate=dict(type="bool", default=True),
        auto_attach=dict(type="bool", default=False),
        auto_propagate=dict(type="bool", default=True),
        description=dict(type="str"),
        multicast_support=dict(type="bool"),
        dns_support=dict(type="bool", default=True),
        purge_tags=dict(type="bool", default=True),
        state=dict(default="present", choices=["present", "absent"]),
        tags=dict(type="dict", aliases=["resource_tags"]),
        transit_gateway_id=dict(type="str"),
        vpn_ecmp_support=dict(type="bool", default=True),
        wait=dict(type="bool", default=True),
        wait_timeout=dict(type="int", default=300),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_one_of=[("description", "transit_gateway_id")],
        supports_check_mode=True,
    )

    return module


def main():
    module = setup_module_object()

    results = dict(changed=False)

    tgw_manager = AnsibleEc2Tgw(module=module, results=results)
    tgw_manager.process()

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: user_v1_info
short_description: Information module for User V1
description:
- Get all User V1.
- Get all users for the Cisco DNA Center System.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  invokeSource:
    description:
    - >
      InvokeSource query parameter. The source that invokes this API. The value of this query parameter must be
      set to "external".
    type: str
  authSource:
    description:
    - >
      AuthSource query parameter. The source that authenticates the user. The value of this query parameter can be
      set to "internal" or "external". If not provided, then all users will be returned in the response.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for User and Roles GetUsersAPIV1
  description: Complete reference of the GetUsersAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-users-api
notes:
  - SDK Method used are
    user_and_roles.UserandRoles.get_users_api_v1,

  - Paths used are
    get /dna/system/api/v1/user,

"""

EXAMPLES = r"""
- name: Get all User V1
  cisco.dnac.user_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    invokeSource: string
    authSource: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "users": [
        {
          "firstName": "string",
          "lastName": "string",
          "authSource": "string",
          "passphraseUpdateTime": "string",
          "roleList": [
            "string"
          ],
          "userId": "string",
          "email": "string",
          "username": "string"
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: route53_info
short_description: Retrieves Route 53 details using AWS methods
version_added: 5.0.0
description:
  - Gets various details related to Route 53 zone, record set or health check details.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  query:
    description:
      - Specifies the query action to take.
    required: true
    choices: [
            'change',
            'checker_ip_range',
            'health_check',
            'hosted_zone',
            'record_sets',
            'reusable_delegation_set',
            ]
    type: str
  change_id:
    description:
      - The ID of the change batch request.
      - The value that you specify here is the value that
        ChangeResourceRecordSets returned in the Id element
        when you submitted the request.
      - Required if O(query=change).
    required: false
    type: str
  hosted_zone_id:
    description:
      - The Hosted Zone ID of the DNS zone.
      - Required if O(query) is set to V(hosted_zone) and O(hosted_zone_method) is set to V(details).
      - Required if O(query) is set to V(record_sets).
    required: false
    type: str
  max_items:
    description:
      - Maximum number of items to return for various get/list requests.
    required: false
    type: int
  next_marker:
    description:
      - "Some requests such as list_command: hosted_zones will return a maximum
        number of entries - EG 100 or the number specified by O(max_items).
        If the number of entries exceeds this maximum another request can be sent
        using the NextMarker entry from the first response to get the next page
        of results."
    required: false
    type: str
  delegation_set_id:
    description:
      - The DNS Zone delegation set ID.
    required: false
    type: str
  start_record_name:
    description:
      - "The first name in the lexicographic ordering of domain names that you want
        the list_command: record_sets to start listing from."
    required: false
    type: str
  type:
    description:
      - The type of DNS record.
    required: false
    choices: [ 'A', 'CNAME', 'MX', 'AAAA', 'TXT', 'PTR', 'SRV', 'SPF', 'CAA', 'NS', 'NAPTR', 'SOA', 'DS' ]
    type: str
  dns_name:
    description:
      - The first name in the lexicographic ordering of domain names that you want
        the list_command to start listing from.
    required: false
    type: str
  resource_id:
    description:
      - The ID/s of the specified resource/s.
      - Required if O(query=health_check) and O(health_check_method=tags).
      - Required if O(query=hosted_zone) and O(hosted_zone_method=tags).
    required: false
    aliases: ['resource_ids']
    type: list
    elements: str
  health_check_id:
    description:
      - The ID of the health check.
      - Required if V(query) is set to V(health_check) and
        O(health_check_method) is set to V(details) or V(status) or V(failure_reason).
    required: false
    type: str
  hosted_zone_method:
    description:
      - "This is used in conjunction with query: hosted_zone.
        It allows for listing details, counts or tags of various
        hosted zone details."
    required: false
    choices: [
        'details',
        'list',
        'list_by_name',
        'count',
        'tags',
        ]
    default: 'list'
    type: str
  health_check_method:
    description:
      - "This is used in conjunction with query: health_check.
        It allows for listing details, counts or tags of various
        health check details."
    required: false
    choices: [
        'list',
        'details',
        'status',
        'failure_reason',
        'count',
        'tags',
        ]
    default: 'list'
    type: str
author:
  - Karen Cheng (@Etherdaemon)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Simple example of listing all hosted zones
- name: List all hosted zones
  amazon.aws.route53_info:
    query: hosted_zone
  register: hosted_zones

# Getting a count of hosted zones
- name: Return a count of all hosted zones
  amazon.aws.route53_info:
    query: hosted_zone
    hosted_zone_method: count
  register: hosted_zone_count

- name: List the first 20 resource record sets in a given hosted zone
  amazon.aws.route53_info:
    profile: account_name
    query: record_sets
    hosted_zone_id: ZZZ1111112222
    max_items: 20
  register: record_sets

- name: List first 20 health checks
  amazon.aws.route53_info:
    query: health_check
    health_check_method: list
    max_items: 20
  register: health_checks

- name: Get health check last failure_reason
  amazon.aws.route53_info:
    query: health_check
    health_check_method: failure_reason
    health_check_id: 00000000-1111-2222-3333-12345678abcd
  register: health_check_failure_reason

- name: Retrieve reusable delegation set details
  amazon.aws.route53_info:
    query: reusable_delegation_set
    delegation_set_id: delegation id
  register: delegation_sets

- name: setup of example for using next_marker
  amazon.aws.route53_info:
    query: hosted_zone
    max_items: 1
  register: first_info

- name: example for using next_marker
  amazon.aws.route53_info:
    query: hosted_zone
    next_marker: "{{ first_info.NextMarker }}"
    max_items: 1
  when: "{{ 'NextMarker' in first_info }}"

- name: retrieve host entries starting with host1.workshop.test.io
  block:
    - name: grab zone id
      amazon.aws.route53_zone:
        zone: "test.io"
      register: AWSINFO

    - name: grab Route53 record information
      amazon.aws.route53_info:
        type: A
        query: record_sets
        hosted_zone_id: "{{ AWSINFO.zone_id }}"
        start_record_name: "host1.workshop.test.io"
      register: RECORDS
"""

RETURN = r"""
resource_record_sets:
    description: A list of resource record sets returned by list_resource_record_sets in boto3.
    returned: when O(query=record_sets)
    type: list
    elements: dict
    contains:
        name:
            description: The name of a record in the specified hosted zone.
            type: str
            sample: 'www.example.com'
        type:
            description: The DNS record type.
            type: str
            sample: 'A'
        ttl:
            description: The resource record cache time to live (TTL), in seconds.
            type: int
            sample: 60
        set_identifier:
            description: An identifier that differentiates among multiple resource record sets that have the same combination of name and type.
            type: str
            sample: 'abcd'
        resource_records:
            description: Information about the resource records.
            type: list
            elements: dict
            contains:
                value:
                    description: The current or new DNS record value.
                    type: str
                    sample: 'ns-12.awsdns-34.com.'
        geo_location:
            description: The specified geographic location for which the Route53 responds to based on location.
            type: dict
            elements: str
            contains:
                continent_code:
                    description: The two-letter code for the continent.
                    type: str
                    sample: 'NA'
                country_code:
                    description: The two-letter code for a country.
                    type: str
                    sample: 'US'
                subdivision_code:
                    description: The two-letter code for a state of the United States
                    type: str
                    sample: 'NY'
    version_added: 4.0.0
    version_added_collection: community.aws
hosted_zones:
    description: A list of hosted zones returned by list_hosted_zones in boto3.
    returned: when O(query=hosted_zone)
    type: list
    elements: dict
    contains:
        id:
            description: The ID of the hosted zone assigned by Amazon Route53 to the hosted zone at the creation time.
            type: str
            sample: '/hostedzone/Z01234567AB1234567890'
        name:
            description: The name of the domain.
            type: str
            sample: 'example.io'
        resource_record_set_count:
            description: The number of resource record sets in the hosted zone.
            type: int
            sample: 3
        caller_reference:
            description: The value specified for CallerReference at the time of hosted zone creation.
            type: str
            sample: '01d0db12-x0x9-12a3-1234-0z000z00zz0z'
        config:
            description: A dict that contains Comment and PrivateZone elements.
            type: dict
            contains:
                comment:
                    description: Any comments that included about in the hosted zone.
                    type: str
                    sample: 'HostedZone created by Route53 Registrar'
                private_zone:
                    description: A value that indicates whether this is a private hosted zone or not.
                    type: bool
                    sample: false
    version_added: 4.0.0
    version_added_collection: community.aws
health_checks:
    description: A list of Route 53 health checks returned by list_health_checks in boto3.
    type: list
    elements: dict
    returned: when O(query=health_check)
    contains:
        id:
            description: The identifier that Amazon Route53 assigned to the health check at the time of creation.
            type: str
            sample: '12345cdc-2cc4-1234-bed2-123456abc1a2'
        health_check_version:
            description: The version of the health check.
            type: str
            sample: 1
        caller_reference:
            description: A unique string that you specified when you created the health check.
            type: str
            sample: '01d0db12-x0x9-12a3-1234-0z000z00zz0z'
        health_check_config:
            description: A dict that contains detailed information about one health check.
            type: dict
            contains:
                disabled:
                    description: Whether Route53 should stop performing health checks on a endpoint.
                    type: bool
                    sample: false
                enable_sni:
                    description: Whether Route53 should send value of FullyQualifiedDomainName to endpoint in client_hello message during TLS negotiation.
                    type: bool
                    sample: true
                failure_threshold:
                    description: The number of consecutive health checks that an endpoint must pass/fail for Route 53 to change current status of endpoint.
                    type: int
                    sample: 3
                fully_qualified_domain_name:
                    description: The fully qualified DNS name of the endpoint on which Route53 performs health checks.
                    type: str
                    sample: 'hello'
                inverted:
                    description: Whether Route53 should invert the status of a health check.
                    type: bool
                    sample: false
                ip_address:
                    description: The IPv4/IPv6 IP address of the endpoint that Route53 should perform health checks on.
                    type: str
                    sample: 192.0.2.44
                measure_latency:
                    description: Whether Route53 should measure latency between health checkers in multiple AWS regions and the endpoint.
                    type: bool
                    sample: false
                port:
                    description: The port of the endpoint that Route53 should perform health checks on.
                    type: int
                    sample: 80
                request_interval:
                    description: The number of seconds between the time that Route53 gets a response from endpoint and the next health check request.
                    type: int
                    sample: 30
                resource_path:
                    description: The path that Route53 requests when performing health checks.
                    type: str
                    sample: '/welcome.html'
                search_string:
                    description: The string that Route 53 uses to search for in the response body from specified resource.
                    type: str
                    sample: 'test-string-to-match'
                type:
                    description: The type of the health check.
                    type: str
                    sample: HTTPS
    version_added: 4.0.0
    version_added_collection: community.aws
checker_ip_ranges:
    description: A list of IP ranges in CIDR format for Amazon Route 53 health checkers.
    returned: when O(query=checker_ip_range)
    type: list
    elements: str
    version_added: 4.1.0
    version_added_collection: community.aws
delegation_sets:
    description: A list of dicts that contains information about the reusable delegation set.
    returned: when O(query=reusable_delegation_set)
    type: list
    elements: dict
    version_added: 4.1.0
    version_added_collection: community.aws
health_check:
    description: A dict of Route53 health check details returned by get_health_check in boto3.
    type: dict
    returned: when O(query=health_check) and O(health_check_method=details)
    contains:
        id:
            description: The identifier that Amazon Route 53 assigned to the health check at the time of creation.
            type: str
            sample: '12345cdc-2cc4-1234-bed2-123456abc1a2'
        health_check_version:
            description: The version of the health check.
            type: str
            sample: 1
        caller_reference:
            description: A unique string that you specified when you created the health check.
            type: str
            sample: '01d0db12-x0x9-12a3-1234-0z000z00zz0z'
        health_check_config:
            description: A dict that contains detailed information about one health check.
            type: dict
            contains:
                disabled:
                    description: Whether Route53 should stop performing health checks on a endpoint.
                    type: bool
                    sample: false
                enable_sni:
                    description: Whether Route53 should send value of FullyQualifiedDomainName to endpoint in client_hello message during TLS negotiation.
                    type: bool
                    sample: true
                failure_threshold:
                    description: The number of consecutive health checks that an endpoint must pass/fail for Route53 to change current status of endpoint.
                    type: int
                    sample: 3
                fully_qualified_domain_name:
                    description: The fully qualified DNS name of the endpoint on which Route53 performs health checks.
                    type: str
                    sample: 'hello'
                inverted:
                    description: Whether Route53 should invert the status of a health check.
                    type: bool
                    sample: false
                ip_address:
                    description: The IPv4/IPv6 IP address of the endpoint that Route53 should perform health checks on.
                    type: str
                    sample: 192.0.2.44
                measure_latency:
                    description: Whether Route53 should measure latency between health checkers in multiple AWS regions and the endpoint.
                    type: bool
                    sample: false
                port:
                    description: The port of the endpoint that Route53 should perform health checks on.
                    type: int
                    sample: 80
                request_interval:
                    description: The number of seconds between the time that Route53 gets a response from endpoint and the next health check request.
                    type: int
                    sample: 30
                resource_path:
                    description: The path that Route53 requests when performing health checks.
                    type: str
                    sample: '/welcome.html'
                search_string:
                    description: The string that Route53 uses to search for in the response body from specified resource.
                    type: str
                    sample: 'test-string-to-match'
                type:
                    description: The type of the health check.
                    type: str
                    sample: HTTPS
    version_added: 4.1.0
    version_added_collection: community.aws
health_check_observations:
    description: A dict of Route53 health check details returned by get_health_check_status and get_health_check_last_failure_reason in boto3.
    type: list
    elements: dict
    returned: when O(query=health_check) and O(health_check_method=status) or O(health_check_method=failure_reason)
    contains:
        ip_address:
            description: The IP address of the Amazon Route 53 health checker that provided the failure reason in StatusReport.
            type: str
            sample: '12.345.67.89'
        region:
            description: The region of the Amazon Route 53 health checker that provided the status in StatusReport.
            type: str
            sample: 'us-west-1'
        status_report:
            description: A complex type that contains the last failure reason and the time of the failed health check.
            type: dict
            contains:
                checked_time:
                    description: The date and time that the health checker performed the health check in ISO 8601 format and Coordinated Universal Time (UTC).
                    type: str
                    sample: '2023-03-08T23:10:08.452000+00:00'
                status:
                    description: A description of the status of the health check endpoint as reported by one of the Amazon Route 53 health checkers.
                    type: str
                    sample: 'Failure: Resolved IP: 12.345.67.89. The connection was closed by the endpoint.'
    version_added: 5.4.0
ResourceRecordSets:
    description: A deprecated CamelCased list of resource record sets returned by list_resource_record_sets in boto3. \
                 This list contains same elements/parameters as it's snake_cased version mentioned above. \
                 This field is deprecated and will be removed in 6.0.0 version release.
    returned: when O(query=record_sets)
    type: list
    elements: dict
HostedZones:
    description: A deprecated CamelCased list of hosted zones returned by list_hosted_zones in boto3. \
                 This list contains same elements/parameters as it's snake_cased version mentioned above. \
                 This field is deprecated and will be removed in 6.0.0 version release.
    returned: when O(query=hosted_zone)
    type: list
    elements: dict
HealthChecks:
    description: A deprecated CamelCased list of Route53 health checks returned by list_health_checks in boto3. \
                 This list contains same elements/parameters as it's snake_cased version mentioned above. \
                 This field is deprecated and will be removed in 6.0.0 version release.
    type: list
    elements: dict
    returned: when O(query=health_check)
CheckerIpRanges:
    description: A deprecated CamelCased list of IP ranges in CIDR format for Amazon Route 53 health checkers.\
                 This list contains same elements/parameters as it's snake_cased version mentioned abobe. \
                 This field is deprecated and will be removed in 6.0.0 version release.
    type: list
    elements: str
    returned: when O(query=checker_ip_range)
DelegationSets:
    description: A deprecated CamelCased list of dicts that contains information about the reusable delegation set. \
                 This list contains same elements/parameters as it's snake_cased version mentioned above. \
                 This field is deprecated and will be removed in 6.0.0 version release.
    type: list
    elements: dict
    returned: when O(query=reusable_delegation_set)
HealthCheck:
    description: A deprecated CamelCased dict of Route53 health check details returned by get_health_check in boto3. \
                 This dict contains same elements/parameters as it's snake_cased version mentioned above. \
                 This field is deprecated and will be removed in 6.0.0 version release.
    type: dict
    returned: when O(query=health_check) and O(health_check_method=details)
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


# Split out paginator to allow for the backoff decorator to function
@AWSRetry.jittered_backoff()
def _paginated_result(paginator_name, **params):
    paginator = client.get_paginator(paginator_name)
    return paginator.paginate(**params).build_full_result()


def get_hosted_zone():
    params = dict()

    if module.params.get("hosted_zone_id"):
        params["Id"] = module.params.get("hosted_zone_id")
    else:
        module.fail_json(msg="Hosted Zone Id is required")

    return client.get_hosted_zone(**params)


def reusable_delegation_set_details():
    params = dict()

    if not module.params.get("delegation_set_id"):
        if module.params.get("max_items"):
            params["MaxItems"] = str(module.params.get("max_items"))

        if module.params.get("next_marker"):
            params["Marker"] = module.params.get("next_marker")

        results = client.list_reusable_delegation_sets(**params)
    else:
        params["DelegationSetId"] = module.params.get("delegation_set_id")
        results = client.get_reusable_delegation_set(**params)

    results["delegation_sets"] = results["DelegationSets"]
    module.deprecate(
        (
            "The 'CamelCase' return values with key 'DelegationSets' is deprecated and will be"
            " replaced by 'snake_case' return values with key 'delegation_sets'.  Both case values"
            " are returned for now."
        ),
        date="2025-01-01",
        collection_name="amazon.aws",
    )

    return results


def list_hosted_zones():
    params = dict()

    # Set PaginationConfig with max_items
    if module.params.get("max_items"):
        params["PaginationConfig"] = dict(MaxItems=module.params.get("max_items"))

    if module.params.get("next_marker"):
        params["Marker"] = module.params.get("next_marker")

    if module.params.get("delegation_set_id"):
        params["DelegationSetId"] = module.params.get("delegation_set_id")

    zones = _paginated_result("list_hosted_zones", **params)["HostedZones"]
    snaked_zones = [camel_dict_to_snake_dict(zone) for zone in zones]

    module.deprecate(
        (
            "The 'CamelCase' return values with key 'HostedZones' and 'list' are deprecated and"
            " will be replaced by 'snake_case' return values with key 'hosted_zones'.  Both case"
            " values are returned for now."
        ),
        date="2025-01-01",
        collection_name="amazon.aws",
    )

    return {
        "HostedZones": zones,
        "list": zones,
        "hosted_zones": snaked_zones,
    }


def list_hosted_zones_by_name():
    params = dict()

    if module.params.get("hosted_zone_id"):
        params["HostedZoneId"] = module.params.get("hosted_zone_id")

    if module.params.get("dns_name"):
        params["DNSName"] = module.params.get("dns_name")

    if module.params.get("max_items"):
        params["MaxItems"] = str(module.params.get("max_items"))

    return client.list_hosted_zones_by_name(**params)


def change_details():
    params = dict()

    if module.params.get("change_id"):
        params["Id"] = module.params.get("change_id")
    else:
        module.fail_json(msg="change_id is required")

    results = client.get_change(**params)
    return results


def checker_ip_range_details():
    results = client.get_checker_ip_ranges()
    results["checker_ip_ranges"] = results["CheckerIpRanges"]
    module.deprecate(
        (
            "The 'CamelCase' return values with key 'CheckerIpRanges' is deprecated and will be"
            " replaced by 'snake_case' return values with key 'checker_ip_ranges'.  Both case values"
            " are returned for now."
        ),
        date="2025-01-01",
        collection_name="amazon.aws",
    )

    return results


def get_count():
    if module.params.get("query") == "health_check":
        results = client.get_health_check_count()
    else:
        results = client.get_hosted_zone_count()

    return results


def get_health_check():
    params = dict()
    results = dict()

    if not module.params.get("health_check_id"):
        module.fail_json(msg="health_check_id is required")
    else:
        params["HealthCheckId"] = module.params.get("health_check_id")

    if module.params.get("health_check_method") == "details":
        results = client.get_health_check(**params)
        results["health_check"] = camel_dict_to_snake_dict(results["HealthCheck"])
        module.deprecate(
            (
                "The 'CamelCase' return values with key 'HealthCheck' is deprecated and will be"
                " replaced by 'snake_case' return values with key 'health_check'.  Both case values are"
                " returned for now."
            ),
            date="2025-01-01",
            collection_name="amazon.aws",
        )

    elif module.params.get("health_check_method") == "failure_reason":
        response = client.get_health_check_last_failure_reason(**params)
        results["health_check_observations"] = [
            camel_dict_to_snake_dict(health_check) for health_check in response["HealthCheckObservations"]
        ]

    elif module.params.get("health_check_method") == "status":
        response = client.get_health_check_status(**params)
        results["health_check_observations"] = [
            camel_dict_to_snake_dict(health_check) for health_check in response["HealthCheckObservations"]
        ]

    return results


def get_resource_tags():
    params = dict()

    if module.params.get("resource_id"):
        params["ResourceIds"] = module.params.get("resource_id")
    else:
        module.fail_json(msg="resource_id or resource_ids is required")

    if module.params.get("query") == "health_check":
        params["ResourceType"] = "healthcheck"
    else:
        params["ResourceType"] = "hostedzone"

    return client.list_tags_for_resources(**params)


def list_health_checks():
    params = dict()

    if module.params.get("next_marker"):
        params["Marker"] = module.params.get("next_marker")

    # Set PaginationConfig with max_items
    if module.params.get("max_items"):
        params["PaginationConfig"] = dict(MaxItems=module.params.get("max_items"))

    health_checks = _paginated_result("list_health_checks", **params)["HealthChecks"]
    snaked_health_checks = [camel_dict_to_snake_dict(health_check) for health_check in health_checks]

    module.deprecate(
        (
            "The 'CamelCase' return values with key 'HealthChecks' and 'list' are deprecated and"
            " will be replaced by 'snake_case' return values with key 'health_checks'.  Both case"
            " values are returned for now."
        ),
        date="2025-01-01",
        collection_name="amazon.aws",
    )

    return {
        "HealthChecks": health_checks,
        "list": health_checks,
        "health_checks": snaked_health_checks,
    }


def record_sets_details():
    params = dict()

    if module.params.get("hosted_zone_id"):
        params["HostedZoneId"] = module.params.get("hosted_zone_id")
    else:
        module.fail_json(msg="Hosted Zone Id is required")

    if module.params.get("start_record_name"):
        params["StartRecordName"] = module.params.get("start_record_name")

    # Check that both params are set if type is applied
    if module.params.get("type") and not module.params.get("start_record_name"):
        module.fail_json(msg="start_record_name must be specified if type is set")

    if module.params.get("type"):
        params["StartRecordType"] = module.params.get("type")

    # Set PaginationConfig with max_items
    if module.params.get("max_items"):
        params["PaginationConfig"] = dict(MaxItems=module.params.get("max_items"))

    record_sets = _paginated_result("list_resource_record_sets", **params)["ResourceRecordSets"]
    snaked_record_sets = [camel_dict_to_snake_dict(record_set) for record_set in record_sets]

    module.deprecate(
        (
            "The 'CamelCase' return values with key 'ResourceRecordSets' and 'list' are deprecated and"
            " will be replaced by 'snake_case' return values with key 'resource_record_sets'."
            "  Both case values are returned for now."
        ),
        date="2025-01-01",
        collection_name="amazon.aws",
    )

    return {
        "ResourceRecordSets": record_sets,
        "list": record_sets,
        "resource_record_sets": snaked_record_sets,
    }


def health_check_details():
    health_check_invocations = {
        "list": list_health_checks,
        "details": get_health_check,
        "status": get_health_check,
        "failure_reason": get_health_check,
        "count": get_count,
        "tags": get_resource_tags,
    }

    results = health_check_invocations[module.params.get("health_check_method")]()
    return results


def hosted_zone_details():
    hosted_zone_invocations = {
        "details": get_hosted_zone,
        "list": list_hosted_zones,
        "list_by_name": list_hosted_zones_by_name,
        "count": get_count,
        "tags": get_resource_tags,
    }

    results = hosted_zone_invocations[module.params.get("hosted_zone_method")]()
    return results


def main():
    global module
    global client

    argument_spec = dict(
        query=dict(
            choices=[
                "change",
                "checker_ip_range",
                "health_check",
                "hosted_zone",
                "record_sets",
                "reusable_delegation_set",
            ],
            required=True,
        ),
        change_id=dict(),
        hosted_zone_id=dict(),
        max_items=dict(type="int"),
        next_marker=dict(),
        delegation_set_id=dict(),
        start_record_name=dict(),
        type=dict(
            type="str",
            choices=["A", "CNAME", "MX", "AAAA", "TXT", "PTR", "SRV", "SPF", "CAA", "NS", "NAPTR", "SOA", "DS"],
        ),
        dns_name=dict(),
        resource_id=dict(type="list", aliases=["resource_ids"], elements="str"),
        health_check_id=dict(),
        hosted_zone_method=dict(choices=["details", "list", "list_by_name", "count", "tags"], default="list"),
        health_check_method=dict(
            choices=[
                "list",
                "details",
                "status",
                "failure_reason",
                "count",
                "tags",
            ],
            default="list",
        ),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ["hosted_zone_method", "health_check_method"],
        ],
        check_boto3=False,
    )

    try:
        client = module.client("route53", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    invocations = {
        "change": change_details,
        "checker_ip_range": checker_ip_range_details,
        "health_check": health_check_details,
        "hosted_zone": hosted_zone_details,
        "record_sets": record_sets_details,
        "reusable_delegation_set": reusable_delegation_set_details,
    }

    results = dict(changed=False)
    try:
        results = invocations[module.params.get("query")]()
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Query failed")

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: event_subscription_email_v1
short_description: Resource module for Event Subscription Email V1
description:
- Manage operations create and update of the resource Event Subscription Email V1.
- Create Email Subscription Endpoint for list of registered events.
- Update Email Subscription Endpoint for list of registered events.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Event Subscription Email's payload.
    elements: dict
    suboptions:
      description:
        description: Description.
        type: str
      filter:
        description: Event Subscription Email's filter.
        suboptions:
          categories:
            description: Categories.
            elements: str
            type: list
          domainsSubdomains:
            description: Event Subscription Email's domainsSubdomains.
            elements: dict
            suboptions:
              domain:
                description: Domain.
                type: str
              subDomains:
                description: Sub Domains.
                elements: str
                type: list
            type: list
          eventIds:
            description: Event Ids.
            elements: str
            type: list
          severities:
            description: Severities.
            elements: int
            type: list
          siteIds:
            description: Site Ids.
            elements: str
            type: list
          sources:
            description: Sources.
            elements: str
            type: list
          types:
            description: Types.
            elements: str
            type: list
        type: dict
      name:
        description: Name.
        type: str
      subscriptionEndpoints:
        description: Event Subscription Email's subscriptionEndpoints.
        elements: dict
        suboptions:
          instanceId:
            description: (From Get Email Subscription Details --> pick InstanceId if
              available).
            type: str
          subscriptionDetails:
            description: Event Subscription Email's subscriptionDetails.
            suboptions:
              connectorType:
                description: Connector Type (Must be EMAIL).
                type: str
              description:
                description: Description.
                type: str
              fromEmailAddress:
                description: Senders Email Address.
                type: str
              name:
                description: Name.
                type: str
              subject:
                description: Email Subject.
                type: str
              toEmailAddresses:
                description: Recipient's Email Addresses (Comma separated).
                elements: str
                type: list
            type: dict
        type: list
      subscriptionId:
        description: Subscription Id (Unique UUID).
        type: str
      version:
        description: Version.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Event Management CreateEmailEventSubscriptionV1
  description: Complete reference of the CreateEmailEventSubscriptionV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-email-event-subscription
- name: Cisco DNA Center documentation for Event Management UpdateEmailEventSubscriptionV1
  description: Complete reference of the UpdateEmailEventSubscriptionV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-email-event-subscription
notes:
  - SDK Method used are
    event_management.EventManagement.create_email_event_subscription_v1,
    event_management.EventManagement.update_email_event_subscription_v1,

  - Paths used are
    post /dna/intent/api/v1/event/subscription/email,
    put /dna/intent/api/v1/event/subscription/email,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.event_subscription_email_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - description: string
      filter:
        categories:
        - string
        domainsSubdomains:
        - domain: string
          subDomains:
          - string
        eventIds:
        - string
        severities:
        - 0
        siteIds:
        - string
        sources:
        - string
        types:
        - string
      name: string
      subscriptionEndpoints:
      - instanceId: string
        subscriptionDetails:
          connectorType: string
          description: string
          fromEmailAddress: string
          name: string
          subject: string
          toEmailAddresses:
          - string
      subscriptionId: string
      version: string

- name: Update all
  cisco.dnac.event_subscription_email_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - description: string
      filter:
        categories:
        - string
        domainsSubdomains:
        - domain: string
          subDomains:
          - string
        eventIds:
        - string
        severities:
        - 0
        siteIds:
        - string
        sources:
        - string
        types:
        - string
      name: string
      subscriptionEndpoints:
      - instanceId: string
        subscriptionDetails:
          connectorType: string
          description: string
          fromEmailAddress: string
          name: string
          subject: string
          toEmailAddresses:
          - string
      subscriptionId: string
      version: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "statusUri": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2016 Matt Davis, <mdavis@ansible.com>
#                    Chris Houseknecht, <house@redhat.com>

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_networkinterface_info

version_added: "0.1.2"

short_description: Get network interface facts

description:
    - Get facts for a specific network interface or all network interfaces within a resource group.

options:
    name:
        description:
            - Only show results for a specific network interface.
        type: str
    resource_group:
        description:
            - Name of the resource group containing the network interface(s). Required when searching by name.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Chris Houseknecht (@chouseknecht)
    - Matt Davis (@nitzmahone)

'''

EXAMPLES = '''
- name: Get facts for one network interface
  azure_rm_networkinterface_info:
    resource_group: myResourceGroup
    name: nic001

- name: Get network interfaces within a resource group
  azure_rm_networkinterface_info:
    resource_group: myResourceGroup

- name: Get network interfaces by tag
  azure_rm_networkinterface_info:
    resource_group: myResourceGroup
    tags:
      - testing
      - foo:bar
'''

RETURN = '''
networkinterfaces:
    description:
        - List of network interface dicts. Each dict contains parameters can be passed to M(azure.azcollection.azure_rm_networkinterface) module.
    type: complex
    returned: always
    contains:
        id:
            description:
                - Id of the network interface.
            type: str
            returned: always
            sample: "/subscriptions/xxxx-xxxxx/resourceGroups/testRG/providers/Microsoft.Network/networkInterfaces/nic01"
        resource_group:
            description:
                - Name of a resource group where the network interface exists.
            type: str
            returned: always
            sample: testRG
        name:
            description:
                - Name of the network interface.
            type: str
            returned: always
            sample: nic01
        location:
            description:
                - Azure location.
            type: str
            returned: always
            sample: eastus
        virtual_network:
            description:
                - An existing virtual network with which the network interface will be associated.
                - It is a dict which contains I(name) and I(resource_group) of the virtual network.
            type: complex
            returned: always
            contains:
                name:
                    description:
                        - The name of the virtual network relate network interface.
                    type: str
                    returned: always
                    sample: vnetnic01
                resource_gorup:
                    description:
                        - Resource groups name that exist on the virtual network.
                    type: str
                    returned: always
                    sample: testRG
                subscription_id:
                    description:
                        - Virtual network Subscription ID.
                    type: str
                    returned: always
                    sample: xxxxxxx-xxxxxxxxxxxxx
                subnet_id:
                    description:
                        - The subnet's ID.
                    type: str
                    returned: always
                    sample: "/subscriptions/xxx-xxxx/resourceGroups/testRG/providers/Microsoft.Network/virtualNetworks/nic01/subnets/sub01"
        subnet:
            description:
                - Name of an existing subnet within the specified virtual network.
            type: str
            returned: always
            sample: sub01
        tags:
            description:
                - Tags of the network interface.
            type: dict
            returned: always
            sample: {key1: value1, key2: value2}
        ip_configurations:
            description:
                - List of IP configurations, if contains multiple configurations.
            type: complex
            returned: always
            contains:
                name:
                    description:
                        - Name of the IP configuration.
                    type: str
                    returned: always
                    sample: default
                private_ip_address:
                    description:
                        - Private IP address for the IP configuration.
                    type: str
                    returned: always
                    sample: 10.10.0.4
                private_ip_allocation_method:
                    description:
                        - Private IP allocation method.
                    type: str
                    returned: always
                    sample: Dynamic
                public_ip_address:
                    description:
                        - Name of the public IP address. None for disable IP address.
                    type: str
                    returned: always
                    sample: null
                public_ip_allocation_method:
                    description:
                        - Public IP allocation method.
                    type: str
                    returned: always
                    sample: null
                load_balancer_backend_address_pools:
                    description:
                        - List of existing load-balancer backend address pools associated with the network interface.
                    type: list
                    returned: always
                    sample: null
                application_gateway_backend_address_pools:
                    description:
                        - List of existing application gateway backend address pools associated with the network interface.
                    version_added: "1.10.0"
                    type: list
                    returned: always
                    sample: null
                primary:
                    description:
                        - Whether the IP configuration is the primary one in the list.
                    type: bool
                    returned: always
                    sample: true
                application_security_groups:
                    description:
                        - List of Application security groups.
                    type: list
                    returned: always
                    sample: ['/subscriptions/<subsid>/resourceGroups/<rg>/providers/Microsoft.Network/applicationSecurityGroups/myASG']
        enable_accelerated_networking:
            description:
                - Specifies whether the network interface should be created with the accelerated networking feature or not.
            type: bool
            returned: always
            sample: false
        create_with_security_group:
            description:
                - Specifies whether a default security group should be be created with the NIC. Only applies when creating a new NIC.
            type: bool
            returned: always
            sample: false
        security_group:
            description:
                - A security group resource ID with which to associate the network interface.
            type: str
            returned: always
            sample: null
        enable_ip_forwarding:
            description:
                - Whether to enable IP forwarding
            type: bool
            returned: always
            sample: false
        dns_servers:
            description:
                - Which DNS servers should the NIC lookup.
                - List of IP addresses.
            type: list
            returned: always
            sample: []
        mac_address:
            description:
                - The MAC address of the network interface.
            type: str
            returned: always
            sample: null
        provisioning_state:
            description:
                - The provisioning state of the network interface.
            type: str
            returned: always
            sample: Succeeded
        dns_settings:
            description:
                - The DNS settings in network interface.
            type: complex
            returned: always
            contains:
                dns_servers:
                    description:
                        - List of DNS servers IP addresses.
                    type: list
                    returned: always
                    sample: []
                applied_dns_servers:
                    description:
                        - If the VM that uses this NIC is part of an Availability Set, then this list will have the union of all DNS servers
                          from all NICs that are part of the Availability Set. This property is what is configured on each of those VMs.
                    type: list
                    returned: always
                    sample: []
                internal_dns_name_label:
                    description:
                        - Relative DNS name for this NIC used for internal communications between VMs in the same virtual network.
                    type: str
                    returned: always
                    sample: null
                internal_fqdn:
                    description:
                        - Fully qualified DNS name supporting internal communications between VMs in the same virtual network.
                    type: str
                    returned: always
                    sample: null
'''  # NOQA
try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase, azure_id_to_dict


AZURE_OBJECT_CLASS = 'NetworkInterface'


def nic_to_dict(nic):
    ip_configurations = [
        dict(
            name=config.name,
            private_ip_address=config.private_ip_address,
            private_ip_allocation_method=config.private_ip_allocation_method,
            primary=config.primary if config.primary else False,
            load_balancer_backend_address_pools=([item.id for item in config.load_balancer_backend_address_pools]
                                                 if config.load_balancer_backend_address_pools else None),
            application_gateway_backend_address_pools=([item.id for item in config.application_gateway_backend_address_pools]
                                                       if config.application_gateway_backend_address_pools else None),
            public_ip_address=config.public_ip_address.id if config.public_ip_address else None,
            public_ip_allocation_method=config.public_ip_address.public_ip_allocation_method if config.public_ip_address else None,
            application_security_groups=([asg.id for asg in config.application_security_groups]
                                         if config.application_security_groups else None)
        ) for config in nic.ip_configurations
    ]
    config = nic.ip_configurations[0] if len(nic.ip_configurations) > 0 else None
    subnet_dict = azure_id_to_dict(config.subnet.id) if config and config.subnet else None
    subnet = subnet_dict.get('subnets') if subnet_dict else None
    virtual_network = dict(
        resource_group=subnet_dict.get('resourceGroups'),
        subnet_id=config.subnet.id if config and config.subnet else None,
        subscription_id=subnet_dict.get('subscriptions'),
        name=subnet_dict.get('virtualNetworks')) if subnet_dict else None
    return dict(
        id=nic.id,
        resource_group=azure_id_to_dict(nic.id).get('resourceGroups'),
        name=nic.name,
        subnet=subnet,
        virtual_network=virtual_network,
        location=nic.location,
        tags=nic.tags,
        security_group=nic.network_security_group.id if nic.network_security_group else None,
        dns_settings=dict(
            dns_servers=nic.dns_settings.dns_servers,
            applied_dns_servers=nic.dns_settings.applied_dns_servers,
            internal_dns_name_label=nic.dns_settings.internal_dns_name_label,
            internal_fqdn=nic.dns_settings.internal_fqdn
        ),
        ip_configurations=ip_configurations,
        mac_address=nic.mac_address,
        enable_ip_forwarding=nic.enable_ip_forwarding,
        provisioning_state=nic.provisioning_state,
        enable_accelerated_networking=nic.enable_accelerated_networking,
        dns_servers=nic.dns_settings.dns_servers,
    )


class AzureRMNetworkInterfaceInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            tags=dict(type='list', elements='str')
        )

        self.results = dict(
            changed=False,
        )

        self.name = None
        self.resource_group = None
        self.tags = None

        super(AzureRMNetworkInterfaceInfo, self).__init__(self.module_arg_spec,
                                                          supports_check_mode=True,
                                                          supports_tags=False,
                                                          facts_module=True
                                                          )

    def exec_module(self, **kwargs):

        is_old_facts = self.module._name == 'azure_rm_networkinterface_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_networkinterface_facts' module has been renamed to 'azure_rm_networkinterface_info'",
                                  version=(2.9, ))

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name and not self.resource_group:
            self.fail("Parameter error: resource group required when filtering by name.")

        results = []

        if self.name:
            results = self.get_item()
        elif self.resource_group:
            results = self.list_resource_group()
        else:
            results = self.list_all()

        if is_old_facts:
            self.results['ansible_facts'] = {
                'azure_networkinterfaces': self.serialize_nics(results)
            }
        self.results['networkinterfaces'] = self.to_dict_list(results)
        return self.results

    def get_item(self):
        self.log('Get properties for {0}'.format(self.name))
        item = None
        try:
            item = self.network_client.network_interfaces.get(self.resource_group, self.name)
        except ResourceNotFoundError:
            pass

        return [item] if item and self.has_tags(item.tags, self.tags) else []

    def list_resource_group(self):
        self.log('List for resource group')
        try:
            response = self.network_client.network_interfaces.list(self.resource_group)
            return [item for item in response if self.has_tags(item.tags, self.tags)]
        except ResourceNotFoundError as exc:
            self.fail("Error listing by resource group {0} - {1}".format(self.resource_group, str(exc)))

    def list_all(self):
        self.log('List all')
        try:
            response = self.network_client.network_interfaces.list_all()
            return [item for item in response if self.has_tags(item.tags, self.tags)]
        except ResourceNotFoundError as exc:
            self.fail("Error listing all - {0}".format(str(exc)))

    def serialize_nics(self, raws):
        return [self.serialize_obj(item, AZURE_OBJECT_CLASS) for item in raws] if raws else []

    def to_dict_list(self, raws):
        return [nic_to_dict(item) for item in raws] if raws else []


def main():
    AzureRMNetworkInterfaceInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2021 Praveen Ghuge (@praveenghuge), Karl Dasan (@karldas30)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_notificationhub
version_added: "1.7.0"
short_description: Manage Notification Hub
description:
    - Create, update and delete instance of Notification Hub.
options:
    resource_group:
        description:
            - Name of the resource group to which the resource belongs.
        required: True
        type: str
    namespace_name:
        description:
            - Name of the namespace in which to create notification hub.
        required: True
        type: str
    name:
        description:
            - Unique name of the Notification Hub.
        type: str
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    sku:
        description:
            - The name of the SKU.
            - Please see L(https://azure.microsoft.com/en-in/pricing/details/notification-hubs/,).
        default: free
        choices:
            - free
            - basic
            - standard
        type: str
    state:
        description:
            - Assert the state of the Notification Hub.
            - Use C(present) to create or update an notification hub and C(absent) to delete it.
        default: present
        choices:
            - absent
            - present
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Praveen Ghuge (@praveenghuge)
    - Karl Dasan (@karldas30)
'''
EXAMPLES = '''

- name: "Create Notification Hub"
  azure_rm_notificationhub:
    resource_group: testgroupans
    location: eastus
    namespace_name: myNamespace
    name: myhub
    tags:
      - a: b
    sku: free

- name: Delete Notification Hub
  azure_rm_notificationhub:
    resource_group: testgroupans
    name: myNamespace
    state: absent

- name: "Create Notification Hub Namespace"
  azure_rm_notificationhub:
    resource_group: testgroupans
    location: eastus
    namespace_name: myNamespace
    tags:
      - a: b
    sku: free

- name: Delete Notification Hub Namespace
  azure_rm_notificationhub:
    resource_group: testgroupans
    namespace_name: myNamespace
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the Notification Hub namesapce or Notification Hub.
    returned: always
    type: dict
    sample: {
        "additional_properties": {},
        "critical": false,
        "data_center": null,
        "enabled": true,
        "location": "eastus2",
        "metric_id": null,
        "name": "testnaedd3d22d3w",
        "namespace_type": "NotificationHub",
        "provisioning_state": "Succeeded",
        "region": null,
        "scale_unit": null,
        "service_bus_endpoint": "https://testnaedd3d22d3w.servicebus.windows.net:443/",
        "sku": { "name":"Free" },
        "tags": {
            "a": "b"
        },
        "type": "Microsoft.NotificationHubs/namespaces"
    }
'''

import time
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.notificationhubs.models import NotificationHubCreateOrUpdateParameters, NamespaceCreateOrUpdateParameters
    from azure.mgmt.notificationhubs.models import Sku
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureNotificationHub(AzureRMModuleBase):

    def __init__(self):
        # define user inputs from playbook

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            namespace_name=dict(type='str', required=True),
            name=dict(type='str'),
            location=dict(type='str'),
            sku=dict(type='str', choices=[
                     'free', 'basic', 'standard'], default='free'),
            state=dict(choices=['present', 'absent'],
                       default='present', type='str'),
        )

        self.resource_group = None
        self.namespace_name = None
        self.name = None
        self.sku = None
        self.location = None
        self.authorizations = None
        self.tags = None
        self.state = None
        self.results = dict(
            changed=False,
            state=dict()
        )

        super(AzureNotificationHub, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                   supports_check_mode=True,
                                                   supports_tags=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        self.results['check_mode'] = self.check_mode

        # retrieve resource group to make sure it exists
        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        results = dict()
        changed = False

        try:
            self.log(
                'Fetching Notification Hub Namespace {0}'.format(self.name))
            namespace = self.notification_hub_client.namespaces.get(
                self.resource_group, self.namespace_name)

            results = namespace_to_dict(namespace)
            if self.name:
                self.log('Fetching Notification Hub {0}'.format(self.name))
                notification_hub = self.notification_hub_client.notification_hubs.get(
                    self.resource_group, self.namespace_name, self.name)
                results = notification_hub_to_dict(
                    notification_hub)
            # don't change anything if creating an existing namespace, but change if deleting it
            if self.state == 'present':
                changed = False

                update_tags, results['tags'] = self.update_tags(
                    results['tags'])

                if update_tags:
                    changed = True
                elif self.namespace_name and not self.name:
                    if self.sku != results['sku']['name'].lower():
                        changed = True

            elif self.state == 'absent':
                changed = True

        except ResourceNotFoundError:
            # the notification hub does not exist so create it
            if self.state == 'present':
                changed = True
            else:
                # you can't delete what is not there
                changed = False

        self.results['changed'] = changed
        if self.name and not changed:
            self.results['state'] = results

        # return the results if your only gathering information
        if self.check_mode:
            return self.results

        if changed:
            if self.state == "present":
                if self.name is None:
                    self.results['state'] = self.create_or_update_namespaces()
                elif self.namespace_name and self.name:
                    self.results['state'] = self.create_or_update_notification_hub()
            elif self.state == "absent":
                # delete Notification Hub
                if self.name is None:
                    self.delete_namespace()
                elif self.namespace_name and self.name:
                    self.delete_notification_hub()
                self.results['state']['status'] = 'Deleted'

        return self.results

    def create_or_update_namespaces(self):
        '''
        create or update namespaces
        '''
        try:
            namespace_params = NamespaceCreateOrUpdateParameters(
                location=self.location,
                namespace_type="NotificationHub",
                sku=Sku(name=self.sku),
                tags=self.tags
            )
            result = self.notification_hub_client.namespaces.create_or_update(
                self.resource_group,
                self.namespace_name,
                namespace_params)

            namespace = self.notification_hub_client.namespaces.get(
                self.resource_group,
                self.namespace_name)

            while namespace.status == "Created":
                time.sleep(30)
                namespace = self.notification_hub_client.namespaces.get(
                    self.resource_group,
                    self.namespace_name,
                )
        except Exception as ex:
            self.fail("Failed to create namespace {0} in resource group {1}: {2}".format(
                self.namespace_name, self.resource_group, str(ex)))
        return namespace_to_dict(result)

    def create_or_update_notification_hub(self):
        '''
        Create or update Notification Hub.
        :return: create or update Notification Hub instance state dictionary
        '''
        try:
            response = self.create_or_update_namespaces()
            params = NotificationHubCreateOrUpdateParameters(
                location=self.location,
                sku=Sku(name=self.sku),
                tags=self.tags
            )
            result = self.notification_hub_client.notification_hubs.create_or_update(
                self.resource_group,
                self.namespace_name,
                self.name,
                params)
            self.log("Response : {0}".format(result))
        except Exception as ex:
            self.fail("Failed to create notification hub {0} in resource group {1}: {2}".format(
                self.name, self.resource_group, str(ex)))
        return notification_hub_to_dict(result)

    def delete_notification_hub(self):
        '''
        Deletes specified notication hub
        :return True
        '''
        self.log("Deleting the notification hub {0}".format(self.name))
        try:
            result = self.notification_hub_client.notification_hubs.delete(
                self.resource_group, self.namespace_name, self.name)
        except Exception as e:
            self.log('Error attempting to delete notification hub.')
            self.fail(
                "Error deleting the notification hub : {0}".format(str(e)))
        return True

    def delete_namespace(self):
        '''
        Deletes specified namespace
        :return True
        '''
        self.log("Deleting the namespace {0}".format(self.namespace_name))
        try:
            result = self.notification_hub_client.namespaces.begin_delete(
                self.resource_group, self.namespace_name)
        except Exception as e:
            self.log('Error attempting to delete namespace.')
            self.fail(
                "Error deleting the namespace : {0}".format(str(e)))
        return True


def notification_hub_to_dict(item):
    # turn notification hub object into a dictionary (serialization)
    notification_hub = item.as_dict()
    result = dict(
        additional_properties=notification_hub.get(
            'additional_properties', {}),
        id=notification_hub.get('id', None),
        name=notification_hub.get('name', None),
        type=notification_hub.get('type', None),
        location=notification_hub.get(
            'location', '').replace(' ', '').lower(),
        tags=notification_hub.get('tags', None),
        provisioning_state=notification_hub.get(
            'provisioning_state', None),
        region=notification_hub.get('region', None),
        metric_id=notification_hub.get('metric_id', None),
        service_bus_endpoint=notification_hub.get(
            'service_bus_endpoint', None),
        scale_unit=notification_hub.get('scale_unit', None),
        enabled=notification_hub.get('enabled', None),
        critical=notification_hub.get('critical', None),
        data_center=notification_hub.get('data_center', None),
        namespace_type=notification_hub.get('namespace_type', None)
    )
    return result


def namespace_to_dict(item):
    # turn notification hub namespace object into a dictionary (serialization)
    namespace = item.as_dict()
    result = dict(
        additional_properties=namespace.get(
            'additional_properties', {}),
        name=namespace.get('name', None),
        type=namespace.get('type', None),
        location=namespace.get(
            'location', '').replace(' ', '').lower(),
        sku=namespace.get("sku"),
        tags=namespace.get('tags', None),
        provisioning_state=namespace.get(
            'provisioning_state', None),
        region=namespace.get('region', None),
        metric_id=namespace.get('metric_id', None),
        service_bus_endpoint=namespace.get(
            'service_bus_endpoint', None),
        scale_unit=namespace.get('scale_unit', None),
        enabled=namespace.get('enabled', None),
        critical=namespace.get('critical', None),
        data_center=namespace.get('data_center', None),
        namespace_type=namespace.get('namespace_type', None)
    )
    return result


def main():
    AzureNotificationHub()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: clients_summary_analytics_v1
short_description: Resource module for Clients Summary Analytics V1
description:
- Manage operation create of the resource Clients Summary Analytics V1.
- >
   Retrieves summary analytics data related to clients while applying complex filtering, aggregate functions, and
   grouping. This API facilitates obtaining consolidated insights into the performance and status of the clients. For
   detailed information about the usage of the API, please refer to the Open API specification document - https
   //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
   clients1-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Clients Summary Analytics's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Clients Summary Analytics's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: int
    type: list
  groupBy:
    description: Group By.
    elements: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  page:
    description: Clients Summary Analytics's page.
    suboptions:
      cursor:
        description: Cursor.
        type: str
      limit:
        description: Limit.
        type: int
      sortBy:
        description: Clients Summary Analytics's sortBy.
        elements: dict
        suboptions:
          name:
            description: Name.
            type: str
          order:
            description: Order.
            type: str
        type: list
    type: dict
  startTime:
    description: Start Time.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Clients RetrievesSummaryAnalyticsDataRelatedToClientsV1
  description: Complete reference of the RetrievesSummaryAnalyticsDataRelatedToClientsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieves-summary-analytics-data-related-to-clients
notes:
  - SDK Method used are
    clients.Clients.retrieves_summary_analytics_data_related_to_clients_v1,

  - Paths used are
    post /dna/data/api/v1/clients/summaryAnalytics,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.clients_summary_analytics_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
    - function: string
      name: string
    attributes:
    - string
    endTime: 0
    filters:
    - key: string
      operator: string
      value: 0
    groupBy:
    - string
    headers: '{{my_headers | from_json}}'
    page:
      cursor: string
      limit: 0
      sortBy:
      - name: string
        order: string
    startTime: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "groups": [
          {
            "id": "string",
            "attributes": [
              {
                "name": "string",
                "value": 0
              }
            ],
            "aggregateAttributes": [
              {
                "name": "string",
                "function": "string",
                "value": 0
              }
            ]
          }
        ]
      },
      "page": {
        "limit": 0,
        "cursor": "string",
        "count": 0,
        "sortBy": [
          {
            "name": "string",
            "order": "string"
          }
        ]
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_domain_permissions_profile
short_description: Manages domain-permissions-profile objects on Checkpoint over Web Services API
description:
  - Manages domain-permissions-profile objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "3.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  permission_type:
    description:
      - The type of the Permissions Profile.
    type: str
    choices: ['read write all', 'read only all', 'customized']
  edit_common_objects:
    description:
      - Define and manage objects in the Check Point database, Network Objects, Services, Custom Application Site, VPN Community, Users, Servers,
        Resources, Time, UserCheck, and Limit.<br>Only a 'Customized' permission-type profile can edit this permission.
    type: bool
  access_control:
    description:
      - Access Control permissions.<br>Only a 'Customized' permission-type profile can edit these permissions.
    type: dict
    suboptions:
      show_policy:
        description:
          - Select to let administrators work with Access Control rules and NAT rules. If not selected, administrators cannot see these rules.
        type: bool
      policy_layers:
        description:
          - Layer editing permissions.<br>Available only if show-policy is set to true.
        type: dict
        suboptions:
          edit_layers:
            description:
              - a "By Software Blades" - Edit Access Control layers that contain the blades enabled in the Permissions Profile.<br>"By
                Selected Profile In A Layer Editor" - Administrators can only edit the layer if the Access Control layer editor gives editing permission to
                their profiles.
            type: str
            choices: ['By Software Blades', 'By Selected Profile In A Layer Editor']
          app_control_and_url_filtering:
            description:
              - Use Application and URL Filtering in Access Control rules.<br>Available only if edit-layers is set to "By Software Blades".
            type: bool
          content_awareness:
            description:
              - Use specified data types in Access Control rules.<br>Available only if edit-layers is set to "By Software Blades".
            type: bool
          firewall:
            description:
              - Work with Access Control and other Software Blades that do not have their own Policies.<br>Available only if edit-layers is
                set to "By Software Blades".
            type: bool
          mobile_access:
            description:
              - Work with Mobile Access rules.<br>Available only if edit-layers is set to "By Software Blades".
            type: bool
      dlp_policy:
        description:
          - Configure DLP rules and Policies.
        type: str
        choices: ['read', 'write', 'disabled']
      geo_control_policy:
        description:
          - Work with Access Control rules that control traffic to and from specified countries.
        type: str
        choices: ['read', 'write', 'disabled']
      nat_policy:
        description:
          - Work with NAT in Access Control rules.
        type: str
        choices: ['read', 'write', 'disabled']
      qos_policy:
        description:
          - Work with QoS Policies and rules.
        type: str
        choices: ['read', 'write', 'disabled']
      access_control_objects_and_settings:
        description:
          - Allow editing of the following objet types, VPN Community, Access Role, Custom application group,Custom application, Custom category,
            Limit, Application - Match Settings, Application Category - Match Settings,Override Categorization, Application and URL filtering blade - Advanced
            Settings, Content Awareness blade - Advanced Settings.
        type: str
        choices: ['read', 'write', 'disabled']
      app_control_and_url_filtering_update:
        description:
          - Install Application and URL Filtering updates.
        type: bool
      install_policy:
        description:
          - Install Access Control Policies.
        type: bool
  endpoint:
    description:
      - Endpoint permissions. Not supported for Multi-Domain Servers.<br>Only a 'Customized' permission-type profile can edit these permissions.
    type: dict
    suboptions:
      manage_policies_and_software_deployment:
        description:
          - The administrator can work with policies, rules and actions.
        type: bool
      edit_endpoint_policies:
        description:
          - Available only if manage-policies-and-software-deployment is set to true.
        type: bool
      policies_installation:
        description:
          - The administrator can install policies on endpoint computers.
        type: bool
      edit_software_deployment:
        description:
          - The administrator can define deployment rules, create packages for export, and configure advanced package settings.<br>Available only
            if manage-policies-and-software-deployment is set to true.
        type: bool
      software_deployment_installation:
        description:
          - The administrator can deploy packages and install endpoint clients.
        type: bool
      allow_executing_push_operations:
        description:
          - The administrator can start operations that the Security Management Server pushes directly to client computers with no policy
            installation required.
        type: bool
      authorize_preboot_users:
        description:
          - The administrator can add and remove the users who are permitted to log on to Endpoint Security client computers with Full Disk Encryption.
        type: bool
      recovery_media:
        description:
          - The administrator can create recovery media on endpoint computers and devices.
        type: bool
      remote_help:
        description:
          - The administrator can use the Remote Help feature to reset user passwords and give access to locked out users.
        type: bool
      reset_computer_data:
        description:
          - The administrator can reset a computer, which deletes all information about the computer from the Security Management Server.
        type: bool
  events_and_reports:
    description:
      - Events and Reports permissions.<br>Only a 'Customized' permission-type profile can edit these permissions.
    type: dict
    suboptions:
      smart_event:
        description:
          - a 'Custom' - Configure SmartEvent permissions.
        type: str
        choices: ['custom', 'app control and url filtering reports only']
      events:
        description:
          - Work with event queries on the Events tab. Create custom event queries.<br>Available only if smart-event is set to 'Custom'.
        type: str
        choices: ['read', 'write', 'disabled']
      policy:
        description:
          - Configure SmartEvent Policy rules and install SmartEvent Policies.<br>Available only if smart-event is set to 'Custom'.
        type: str
        choices: ['read', 'write', 'disabled']
      reports:
        description:
          - Create and run SmartEvent reports.<br>Available only if smart-event is set to 'Custom'.
        type: bool
  gateways:
    description:
      - Gateways permissions. <br>Only a 'Customized' permission-type profile can edit these permissions.
    type: dict
    suboptions:
      smart_update:
        description:
          - Install, update and delete Check Point licenses. This includes permissions to use SmartUpdate to manage licenses.
        type: str
        choices: ['read', 'write', 'disabled']
      lsm_gw_db:
        description:
          - Access to objects defined in LSM gateway tables. These objects are managed in the SmartProvisioning GUI or LSMcli
            command-line.<br>Note, 'Write' permission on lsm-gw-db allows administrator to run a script on SmartLSM gateway in Expert mode.
        type: str
        choices: ['read', 'write', 'disabled']
      manage_provisioning_profiles:
        description:
          - Administrator can add, edit, delete, and assign provisioning profiles to gateways (both LSM and non-LSM).<br>Available for edit only
            if lsm-gw-db is set with 'Write' permission.<br>Note, 'Read' permission on lsm-gw-db enables 'Read' permission for manage-provisioning-profiles.
        type: str
        choices: ['read', 'write', 'disabled']
      vsx_provisioning:
        description:
          - Create and configure Virtual Systems and other VSX virtual objects.
        type: bool
      system_backup:
        description:
          - Backup Security Gateways.
        type: bool
      system_restore:
        description:
          - Restore Security Gateways from saved backups.
        type: bool
      open_shell:
        description:
          - Use the SmartConsole CLI to run commands.
        type: bool
      run_one_time_script:
        description:
          - Run user scripts from the command line.
        type: bool
      run_repository_script:
        description:
          - Run scripts from the repository.
        type: bool
      manage_repository_scripts:
        description:
          - Add, change and remove scripts in the repository.
        type: str
        choices: ['read', 'write', 'disabled']
  management:
    description:
      - Management permissions.
    type: dict
    suboptions:
      cme_operations:
        description:
          - Permission to read / edit the Cloud Management Extension (CME) configuration.<br>Not supported for Multi-Domain Servers.
        type: str
        choices: ['read', 'write', 'disabled']
      manage_admins:
        description:
          - Controls the ability to manage Administrators, Permission Profiles, Trusted clients,API settings and Policy settings.<br>Only a "Read
            Write All" permission-type profile can edit this permission.<br>Not supported for Multi-Domain Servers.
        type: bool
      management_api_login:
        description:
          - Permission to log in to the Security Management Server and run API commands using thesetools, mgmt_cli (Linux and Windows binaries),
            Gaia CLI (clish) and Web Services (REST). Useful if you want to prevent administrators from running automatic scripts on the Management.<br>Note,
            This permission is not required to run commands from within the API terminal in SmartConsole.<br>Not supported for Multi-Domain Servers.
        type: bool
      manage_sessions:
        description:
          - Lets you disconnect, discard, publish, or take over other administrator sessions.<br>Only a "Read Write All" permission-type profile
            can edit this permission.
        type: bool
      high_availability_operations:
        description:
          - Configure and work with Domain High Availability.<br>Only a 'Customized' permission-type profile can edit this permission.
        type: bool
      approve_or_reject_sessions:
        description:
          - Approve / reject other sessions.
        type: bool
      publish_sessions:
        description:
          - Allow session publishing without an approval.
        type: bool
      manage_integration_with_cloud_services:
        description:
          - Manage integration with Cloud Services.
        type: bool
  monitoring_and_logging:
    description:
      - Monitoring and Logging permissions.<br>'Customized' permission-type profile can edit all these permissions. "Read Write All" permission-type
        can edit only dlp-logs-including-confidential-fields and manage-dlp-messages permissions.
    type: dict
    suboptions:
      monitoring:
        description:
          - See monitoring views and reports.
        type: str
        choices: ['read', 'write', 'disabled']
      management_logs:
        description:
          - See Multi-Domain Server audit logs.
        type: str
        choices: ['read', 'write', 'disabled']
      track_logs:
        description:
          - Use the log tracking features in SmartConsole.
        type: str
        choices: ['read', 'write', 'disabled']
      app_and_url_filtering_logs:
        description:
          - Work with Application and URL Filtering logs.
        type: bool
      https_inspection_logs:
        description:
          - See logs generated by HTTPS Inspection.
        type: bool
      packet_capture_and_forensics:
        description:
          - See logs generated by the IPS and Forensics features.
        type: bool
      show_packet_capture_by_default:
        description:
          - Enable packet capture by default.
        type: bool
      identities:
        description:
          - Show user and computer identity information in logs.
        type: bool
      show_identities_by_default:
        description:
          - Show user and computer identity information in logs by default.
        type: bool
      dlp_logs_including_confidential_fields:
        description:
          - Show DLP logs including confidential fields.
        type: bool
      manage_dlp_messages:
        description:
          - View/Release/Discard DLP messages.<br>Available only if dlp-logs-including-confidential-fields is set to true.
        type: bool
  threat_prevention:
    description:
      - Threat Prevention permissions.<br>Only a 'Customized' permission-type profile can edit these permissions.
    type: dict
    suboptions:
      policy_layers:
        description:
          - Configure Threat Prevention Policy rules.<br>Note, To have policy-layers permissions you must set policy-exceptionsand profiles
            permissions. To have 'Write' permissions for policy-layers, policy-exceptions must be set with 'Write' permission as well.
        type: str
        choices: ['read', 'write', 'disabled']
      edit_layers:
        description:
          - a 'ALL' -  Gives permission to edit all layers.<br>"By Selected Profile In A Layer Editor" -  Administrators can only edit the layer
            if the Threat Prevention layer editor gives editing permission to their profiles.<br>Available only if policy-layers is set to 'Write'.
        type: str
        choices: ['By Selected Profile In A Layer Editor', 'All']
      edit_settings:
        description:
          - Work with general Threat Prevention settings.
        type: bool
      policy_exceptions:
        description:
          - Configure exceptions to Threat Prevention rules.<br>Note, To have policy-exceptions you must set the protections permission.
        type: str
        choices: ['read', 'write', 'disabled']
      profiles:
        description:
          - Configure Threat Prevention profiles.
        type: str
        choices: ['read', 'write', 'disabled']
      protections:
        description:
          - Work with malware protections.
        type: str
        choices: ['read', 'write', 'disabled']
      install_policy:
        description:
          - Install Policies.
        type: bool
      ips_update:
        description:
          - Update IPS protections.<br>Note, You do not have to log into the User Center to receive IPS updates.
        type: bool
  others:
    description:
      - Additional permissions.<br>Only a 'Customized' permission-type profile can edit these permissions.
    type: dict
    suboptions:
      client_certificates:
        description:
          - Create and manage client certificates for Mobile Access.
        type: bool
      edit_cp_users_db:
        description:
          - Work with user accounts and groups.
        type: bool
      https_inspection:
        description:
          - Enable and configure HTTPS Inspection rules.
        type: str
        choices: ['read', 'write', 'disabled']
      ldap_users_db:
        description:
          - Work with the LDAP database and user accounts, groups and OUs.
        type: str
        choices: ['read', 'write', 'disabled']
      user_authority_access:
        description:
          - Work with Check Point User Authority authentication.
        type: str
        choices: ['read', 'write', 'disabled']
      user_device_mgmt_conf:
        description:
          - Gives access to the UDM (User & Device Management) web-based application that handles security challenges in a "bring your own device"
            (BYOD) workspace.
        type: str
        choices: ['read', 'write', 'disabled']
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-domain-permissions-profile
  cp_mgmt_domain_permissions_profile:
    name: customized profile
    state: present

- name: set-domain-permissions-profile
  cp_mgmt_domain_permissions_profile:
    access_control:
      policy_layers: By Selected Profile In A Layer Editor
    name: read profile
    permission_type: customized
    state: present

- name: delete-domain-permissions-profile
  cp_mgmt_domain_permissions_profile:
    name: profile
    state: absent
"""

RETURN = """
cp_mgmt_domain_permissions_profile:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        permission_type=dict(
            type="str",
            choices=["read write all", "read only all", "customized"],
        ),
        edit_common_objects=dict(type="bool"),
        access_control=dict(
            type="dict",
            options=dict(
                show_policy=dict(type="bool"),
                policy_layers=dict(
                    type="dict",
                    options=dict(
                        edit_layers=dict(
                            type="str",
                            choices=[
                                "By Software Blades",
                                "By Selected Profile In A Layer Editor",
                            ],
                        ),
                        app_control_and_url_filtering=dict(type="bool"),
                        content_awareness=dict(type="bool"),
                        firewall=dict(type="bool"),
                        mobile_access=dict(type="bool"),
                    ),
                ),
                dlp_policy=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                geo_control_policy=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                nat_policy=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                qos_policy=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                access_control_objects_and_settings=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                app_control_and_url_filtering_update=dict(type="bool"),
                install_policy=dict(type="bool"),
            ),
        ),
        endpoint=dict(
            type="dict",
            options=dict(
                manage_policies_and_software_deployment=dict(type="bool"),
                edit_endpoint_policies=dict(type="bool"),
                policies_installation=dict(type="bool"),
                edit_software_deployment=dict(type="bool"),
                software_deployment_installation=dict(type="bool"),
                allow_executing_push_operations=dict(type="bool"),
                authorize_preboot_users=dict(type="bool"),
                recovery_media=dict(type="bool"),
                remote_help=dict(type="bool"),
                reset_computer_data=dict(type="bool"),
            ),
        ),
        events_and_reports=dict(
            type="dict",
            options=dict(
                smart_event=dict(
                    type="str",
                    choices=[
                        "custom",
                        "app control and url filtering reports only",
                    ],
                ),
                events=dict(type="str", choices=["read", "write", "disabled"]),
                policy=dict(type="str", choices=["read", "write", "disabled"]),
                reports=dict(type="bool"),
            ),
        ),
        gateways=dict(
            type="dict",
            options=dict(
                smart_update=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                lsm_gw_db=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                manage_provisioning_profiles=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                vsx_provisioning=dict(type="bool"),
                system_backup=dict(type="bool"),
                system_restore=dict(type="bool"),
                open_shell=dict(type="bool"),
                run_one_time_script=dict(type="bool"),
                run_repository_script=dict(type="bool"),
                manage_repository_scripts=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
            ),
        ),
        management=dict(
            type="dict",
            options=dict(
                cme_operations=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                manage_admins=dict(type="bool"),
                management_api_login=dict(type="bool"),
                manage_sessions=dict(type="bool"),
                high_availability_operations=dict(type="bool"),
                approve_or_reject_sessions=dict(type="bool"),
                publish_sessions=dict(type="bool"),
                manage_integration_with_cloud_services=dict(type="bool"),
            ),
        ),
        monitoring_and_logging=dict(
            type="dict",
            options=dict(
                monitoring=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                management_logs=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                track_logs=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                app_and_url_filtering_logs=dict(type="bool"),
                https_inspection_logs=dict(type="bool"),
                packet_capture_and_forensics=dict(type="bool"),
                show_packet_capture_by_default=dict(type="bool"),
                identities=dict(type="bool"),
                show_identities_by_default=dict(type="bool"),
                dlp_logs_including_confidential_fields=dict(type="bool"),
                manage_dlp_messages=dict(type="bool"),
            ),
        ),
        threat_prevention=dict(
            type="dict",
            options=dict(
                policy_layers=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                edit_layers=dict(
                    type="str",
                    choices=["By Selected Profile In A Layer Editor", "All"],
                ),
                edit_settings=dict(type="bool"),
                policy_exceptions=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                profiles=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                protections=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                install_policy=dict(type="bool"),
                ips_update=dict(type="bool"),
            ),
        ),
        others=dict(
            type="dict",
            options=dict(
                client_certificates=dict(type="bool"),
                edit_cp_users_db=dict(type="bool"),
                https_inspection=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                ldap_users_db=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                user_authority_access=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
                user_device_mgmt_conf=dict(
                    type="str", choices=["read", "write", "disabled"]
                ),
            ),
        ),
        tags=dict(type="list", elements="str"),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "domain-permissions-profile"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()
